@@ -19,7 +19,7 @@
 
 
 /* Starting MTU value for the DTLS BIO filter */
-static int mtu = 1440;
+static int mtu = 1200;
 void janus_dtls_bio_filter_set_mtu(int start_mtu) {
 	if(start_mtu < 0) {
 		JANUS_LOG(LOG_ERR, "Invalid MTU...\n");
@@ -129,6 +129,11 @@ int janus_dtls_bio_filter_free(BIO *bio) {
 int janus_dtls_bio_filter_write(BIO *bio, const char *in, int inl) {
 	JANUS_LOG(LOG_HUGE, "janus_dtls_bio_filter_write: %p, %d\n", in, inl);
 	/* Forward data to the write BIO */
+	if(inl <= 0) {
+		/* ... unless the size is negative or zero */
+		JANUS_LOG(LOG_WARN, "janus_dtls_bio_filter_write failed: negative size (%d)\n", inl);
+		return inl;
+	}
 #if JANUS_USE_OPENSSL_PRE_1_1_API
 	long ret = BIO_write(bio->next_bio, in, inl);
 #else
