@@ -26,24 +26,29 @@ static GThread *events_thread;
 void *janus_events_thread(void *data);
 
 int janus_events_init(gboolean enabled, char *server_name, GHashTable *handlers) {
-	/* We setup a thread for passing events to the handlers */
-	GError *error = NULL;
-	events_thread = g_thread_try_new("janus events thread", janus_events_thread, NULL, &error);
-	if(error != NULL) {
-		JANUS_LOG(LOG_ERR, "Got error %d (%s) trying to launch the Events handler thread...\n", error->code, error->message ? error->message : "??");
-		return -1;
-	}
-	if(server_name != NULL)
-		server = g_strdup(server_name);
-	events = g_async_queue_new();
-	eventhandlers = handlers;
 	eventsenabled = enabled;
+	if(eventsenabled) {
+		events = g_async_queue_new();
+		if(server_name != NULL)
+			server = g_strdup(server_name);
+		eventhandlers = handlers;
+		/* We setup a thread for passing events to the handlers */
+		GError *error = NULL;
+		events_thread = g_thread_try_new("janus events thread", janus_events_thread, NULL, &error);
+		if(error != NULL) {
+			eventsenabled = FALSE;
+			g_free(server);
+			g_async_queue_unref(events);
+			JANUS_LOG(LOG_ERR, "Got error %d (%s) trying to launch the Events handler thread...\n", error->code, error->message ? error->message : "??");
+			return -1;
+		}
+	}
 	return 0;
 }
 
 void janus_events_deinit(void) {
 	eventsenabled = FALSE;
-	if (events != NULL) {
+	if(events != NULL) {
 		g_async_queue_push(events, &exit_event);
 	}
 	if(events_thread != NULL) {
@@ -69,10 +74,16 @@ void janus_events_notify_handlers(int type, guint64 session_id, ...) {
 		if(type == JANUS_EVENT_TYPE_MEDIA || type == JANUS_EVENT_TYPE_WEBRTC) {
 			/* These events allocate a json_t object for their data, skip some arguments and unref it */
 			va_arg(args, guint64);
+			va_arg(args, char *);
 			json_t *body = va_arg(args, json_t *);
 			json_decref(body);
 		} else if(type == JANUS_EVENT_TYPE_CORE) {
-			/* Core events also allocate a json_t object for its data, unref it */
+			/* Core events also allocate a json_t object for their data, unref it */
+			json_t *body = va_arg(args, json_t *);
+			json_decref(body);
+		} else if(type == JANUS_EVENT_TYPE_EXTERNAL) {
+			/* Admin API originated external events also allocate a json_t object for their data, unref it */
+			va_arg(args, char *);
 			json_t *body = va_arg(args, json_t *);
 			json_decref(body);
 		} else if(type == JANUS_EVENT_TYPE_SESSION) {
@@ -85,6 +96,7 @@ void janus_events_notify_handlers(int type, guint64 session_id, ...) {
 			/* Plugin originated events also allocate a json_t object for the plugin data, skip some arguments and unref it */
 			va_arg(args, guint64);
 			va_arg(args, char *);
+			va_arg(args, char *);
 			json_t *data = va_arg(args, json_t *);
 			json_decref(data);
 		} else if(type == JANUS_EVENT_TYPE_TRANSPORT) {
@@ -104,7 +116,8 @@ void janus_events_notify_handlers(int type, guint64 session_id, ...) {
 		json_object_set_new(event, "emitter", json_string(server));
 	json_object_set_new(event, "type", json_integer(type));
 	json_object_set_new(event, "timestamp", json_integer(janus_get_real_time()));
-	if(type != JANUS_EVENT_TYPE_CORE) {			/* Core events don't have a session ID */
+	if(type != JANUS_EVENT_TYPE_CORE && type != JANUS_EVENT_TYPE_EXTERNAL) {
+		/* Core and Admin API originated events don't have a session ID */
 		if(session_id == 0 && (type == JANUS_EVENT_TYPE_PLUGIN || type == JANUS_EVENT_TYPE_TRANSPORT)) {
 			/* ... but plugin/transport events may not have one either */
 		} else {
@@ -123,7 +136,7 @@ void janus_events_notify_handlers(int type, guint64 session_id, ...) {
 			json_object_set_new(body, "name", json_string(name));
 			json_t *transport = va_arg(args, json_t *);
 			if(transport != NULL)
-				json_object_set(body, "transport", transport);
+				json_object_set_new(body, "transport", transport);
 			break;
 		}
 		case JANUS_EVENT_TYPE_HANDLE: {
@@ -138,14 +151,21 @@ void janus_events_notify_handlers(int type, guint64 session_id, ...) {
 			/* Handle-related events may include an opaque ID provided by who's using the plugin:
 			 * in event handlers, it may be useful for inter-handle mappings or other things */
 			char *opaque_id = va_arg(args, char *);
-			if(opaque_id != NULL)
+			if(opaque_id != NULL) {
+				json_object_set_new(event, "opaque_id", json_string(opaque_id));
+				/* We add it to the body as well for backwards compatbility, as
+				 * that's the only place we had the opaque_id present before */
 				json_object_set_new(body, "opaque_id", json_string(opaque_id));
+			}
 			break;
 		}
 		case JANUS_EVENT_TYPE_JSEP: {
 			/* For JSEP-related events, there's the handle ID, whether the SDP is local or remote, the JSEP type and the SDP itself */
 			guint64 handle_id = va_arg(args, guint64);
 			json_object_set_new(event, "handle_id", json_integer(handle_id));
+			char *opaque_id = va_arg(args, char *);
+			if(opaque_id != NULL)
+				json_object_set_new(event, "opaque_id", json_string(opaque_id));
 			char *owner = va_arg(args, char *);
 			json_object_set_new(body, "owner", json_string(owner));
 			json_t *jsep = json_object();
@@ -161,6 +181,9 @@ void janus_events_notify_handlers(int type, guint64 session_id, ...) {
 			/* For WebRTC and media-related events, there's the handle ID and a json_t object with info on what happened */
 			guint64 handle_id = va_arg(args, guint64);
 			json_object_set_new(event, "handle_id", json_integer(handle_id));
+			char *opaque_id = va_arg(args, char *);
+			if(opaque_id != NULL)
+				json_object_set_new(event, "opaque_id", json_string(opaque_id));
 			/* The body is what we get from the event */
 			body = va_arg(args, json_t *);
 			break;
@@ -170,10 +193,13 @@ void janus_events_notify_handlers(int type, guint64 session_id, ...) {
 			guint64 handle_id = va_arg(args, guint64);
 			if(handle_id > 0)	/* Plugins and transports may not specify a session and handle ID for out of context events */
 				json_object_set_new(event, "handle_id", json_integer(handle_id));
+			char *opaque_id = va_arg(args, char *);
+			if(opaque_id != NULL)
+				json_object_set_new(event, "opaque_id", json_string(opaque_id));
 			char *name = va_arg(args, char *);
 			json_object_set_new(body, "plugin", json_string(name));
 			json_t *data = va_arg(args, json_t *);
-			json_object_set(body, "data", data);
+			json_object_set_new(body, "data", data);
 			break;
 		}
 		case JANUS_EVENT_TYPE_TRANSPORT: {
@@ -185,7 +211,7 @@ void janus_events_notify_handlers(int type, guint64 session_id, ...) {
 			g_snprintf(id, sizeof(id), "%p", instance);
 			json_object_set_new(body, "id", json_string(id));
 			json_t *data = va_arg(args, json_t *);
-			json_object_set(body, "data", data);
+			json_object_set_new(body, "data", data);
 			break;
 		}
 		case JANUS_EVENT_TYPE_CORE: {
@@ -193,6 +219,13 @@ void janus_events_notify_handlers(int type, guint64 session_id, ...) {
 			body = va_arg(args, json_t *);
 			break;
 		}
+		case JANUS_EVENT_TYPE_EXTERNAL: {
+			char *schema = va_arg(args, char *);
+			json_object_set_new(body, "schema", json_string(schema));
+			json_t *data = va_arg(args, json_t *);
+			json_object_set_new(body, "data", data);
+			break;
+		}
 		default:
 			JANUS_LOG(LOG_WARN, "Unknown event type '%d'\n", type);
 			json_decref(event);
@@ -203,6 +236,10 @@ void janus_events_notify_handlers(int type, guint64 session_id, ...) {
 	json_object_set_new(event, "event", body);
 	va_end(args);
 
+	if(!eventsenabled) {
+		json_decref(event);
+		return;
+	}
 	/* Enqueue the event */
 	g_async_queue_push(events, event);
 }
@@ -239,6 +276,12 @@ void *janus_events_thread(void *data) {
 		json_decref(event);
 	}
 
-	JANUS_LOG(LOG_VERB, "Leaving EchoTest handler thread\n");
+	/* Cleanup pending events */
+	while((event = g_async_queue_try_pop(events)) != NULL) {
+		if(event != &exit_event)
+			json_decref(event);
+	}
+
+	JANUS_LOG(LOG_VERB, "Leaving Events handler thread\n");
 	return NULL;
 }
