@@ -42,6 +42,7 @@ const char *janus_sampleevh_get_name(void);
 const char *janus_sampleevh_get_author(void);
 const char *janus_sampleevh_get_package(void);
 void janus_sampleevh_incoming_event(json_t *event);
+json_t *janus_sampleevh_handle_request(json_t *request);
 
 /* Event handler setup */
 static janus_eventhandler janus_sampleevh =
@@ -58,6 +59,7 @@ static janus_eventhandler janus_sampleevh =
 		.get_package = janus_sampleevh_get_package,
 		
 		.incoming_event = janus_sampleevh_incoming_event,
+		.handle_request = janus_sampleevh_handle_request,
 
 		.events_mask = JANUS_EVENT_TYPE_NONE
 	);
@@ -73,6 +75,7 @@ janus_eventhandler *create(void) {
 static volatile gint initialized = 0, stopping = 0;
 static GThread *handler_thread;
 static void *janus_sampleevh_handler(void *data);
+static janus_mutex evh_mutex;
 
 /* Queue of events to handle */
 static GAsyncQueue *events = NULL;
@@ -95,6 +98,84 @@ static size_t janus_sampleehv_write_data(void *buffer, size_t size, size_t nmemb
 	return size*nmemb;
 }
 
+
+/* Parameter validation (for tweaking via Admin API) */
+static struct janus_json_parameter request_parameters[] = {
+	{"request", JSON_STRING, JANUS_JSON_PARAM_REQUIRED}
+};
+static struct janus_json_parameter tweak_parameters[] = {
+	{"events", JSON_STRING, 0},
+	{"grouping", JANUS_JSON_BOOL, 0},
+	{"backend", JSON_STRING, 0},
+	{"backend_user", JSON_STRING, 0},
+	{"backend_pwd", JSON_STRING, 0},
+	{"max_retransmissions", JSON_INTEGER, JANUS_JSON_PARAM_POSITIVE},
+	{"retransmissions_backoff", JSON_INTEGER, JANUS_JSON_PARAM_POSITIVE}
+};
+/* Error codes (for the tweaking via Admin API */
+#define JANUS_SAMPLEEVH_ERROR_INVALID_REQUEST		411
+#define JANUS_SAMPLEEVH_ERROR_MISSING_ELEMENT		412
+#define JANUS_SAMPLEEVH_ERROR_INVALID_ELEMENT		413
+#define JANUS_SAMPLEEVH_ERROR_UNKNOWN_ERROR			499
+
+
+/* Helper method to change the events mask */
+static void janus_sampleevh_edit_events_mask(const char *list) {
+	if(!list)
+		return;
+	janus_flags mask;
+	janus_flags_reset(&mask);
+	if(!strcasecmp(list, "none")) {
+		/* Don't subscribe to anything at all */
+		janus_flags_reset(&mask);
+	} else if(!strcasecmp(list, "all")) {
+		/* Subscribe to everything */
+		janus_flags_set(&mask, JANUS_EVENT_TYPE_ALL);
+	} else {
+		/* Check what we need to subscribe to */
+		janus_flags_reset(&mask);
+		gchar **subscribe = g_strsplit(list, ",", -1);
+		if(subscribe != NULL) {
+			gchar *index = subscribe[0];
+			if(index != NULL) {
+				int i=0;
+				while(index != NULL) {
+					while(isspace(*index))
+						index++;
+					if(strlen(index)) {
+						if(!strcasecmp(index, "sessions")) {
+							janus_flags_set(&mask, JANUS_EVENT_TYPE_SESSION);
+						} else if(!strcasecmp(index, "handles")) {
+							janus_flags_set(&mask, JANUS_EVENT_TYPE_HANDLE);
+						} else if(!strcasecmp(index, "jsep")) {
+							janus_flags_set(&mask, JANUS_EVENT_TYPE_JSEP);
+						} else if(!strcasecmp(index, "webrtc")) {
+							janus_flags_set(&mask, JANUS_EVENT_TYPE_WEBRTC);
+						} else if(!strcasecmp(index, "media")) {
+							janus_flags_set(&mask, JANUS_EVENT_TYPE_MEDIA);
+						} else if(!strcasecmp(index, "plugins")) {
+							janus_flags_set(&mask, JANUS_EVENT_TYPE_PLUGIN);
+						} else if(!strcasecmp(index, "transports")) {
+							janus_flags_set(&mask, JANUS_EVENT_TYPE_TRANSPORT);
+						} else if(!strcasecmp(index, "core")) {
+							janus_flags_set(&mask, JANUS_EVENT_TYPE_CORE);
+						} else if(!strcasecmp(index, "external")) {
+							janus_flags_set(&mask, JANUS_EVENT_TYPE_EXTERNAL);
+						} else {
+							JANUS_LOG(LOG_WARN, "Unknown event type '%s'\n", index);
+						}
+					}
+					i++;
+					index = subscribe[i];
+				}
+			}
+			g_strfreev(subscribe);
+		}
+	}
+	memcpy(&janus_sampleevh.events_mask, &mask, sizeof(janus_flags));
+}
+
+
 /* Plugin implementation */
 int janus_sampleevh_init(const char *config_path) {
 	if(g_atomic_int_get(&stopping)) {
@@ -156,52 +237,8 @@ int janus_sampleevh_init(const char *config_path) {
 				}
 				/* Which events should we subscribe to? */
 				item = janus_config_get_item_drilldown(config, "general", "events");
-				if(item && item->value) {
-					if(!strcasecmp(item->value, "none")) {
-						/* Don't subscribe to anything at all */
-						janus_flags_reset(&janus_sampleevh.events_mask);
-					} else if(!strcasecmp(item->value, "all")) {
-						/* Subscribe to everything */
-						janus_flags_set(&janus_sampleevh.events_mask, JANUS_EVENT_TYPE_ALL);
-					} else {
-						/* Check what we need to subscribe to */
-						gchar **subscribe = g_strsplit(item->value, ",", -1);
-						if(subscribe != NULL) {
-							gchar *index = subscribe[0];
-							if(index != NULL) {
-								int i=0;
-								while(index != NULL) {
-									while(isspace(*index))
-										index++;
-									if(strlen(index)) {
-										if(!strcasecmp(index, "sessions")) {
-											janus_flags_set(&janus_sampleevh.events_mask, JANUS_EVENT_TYPE_SESSION);
-										} else if(!strcasecmp(index, "handles")) {
-											janus_flags_set(&janus_sampleevh.events_mask, JANUS_EVENT_TYPE_HANDLE);
-										} else if(!strcasecmp(index, "jsep")) {
-											janus_flags_set(&janus_sampleevh.events_mask, JANUS_EVENT_TYPE_JSEP);
-										} else if(!strcasecmp(index, "webrtc")) {
-											janus_flags_set(&janus_sampleevh.events_mask, JANUS_EVENT_TYPE_WEBRTC);
-										} else if(!strcasecmp(index, "media")) {
-											janus_flags_set(&janus_sampleevh.events_mask, JANUS_EVENT_TYPE_MEDIA);
-										} else if(!strcasecmp(index, "plugins")) {
-											janus_flags_set(&janus_sampleevh.events_mask, JANUS_EVENT_TYPE_PLUGIN);
-										} else if(!strcasecmp(index, "transports")) {
-											janus_flags_set(&janus_sampleevh.events_mask, JANUS_EVENT_TYPE_TRANSPORT);
-										} else if(!strcasecmp(index, "core")) {
-											janus_flags_set(&janus_sampleevh.events_mask, JANUS_EVENT_TYPE_CORE);
-										} else {
-											JANUS_LOG(LOG_WARN, "Unknown event type '%s'\n", index);
-										}
-									}
-									i++;
-									index = subscribe[i];
-								}
-							}
-							g_strfreev(subscribe);
-						}
-					}
-				}
+				if(item && item->value)
+					janus_sampleevh_edit_events_mask(item->value);
 				/* Is grouping of events ok? */
 				item = janus_config_get_item_drilldown(config, "general", "grouping");
 				if(item && item->value)
@@ -225,7 +262,8 @@ int janus_sampleevh_init(const char *config_path) {
 
 	/* Initialize the events queue */
 	events = g_async_queue_new_full((GDestroyNotify) janus_sampleevh_event_free);
-	
+	janus_mutex_init(&evh_mutex);
+
 	g_atomic_int_set(&initialized, 1);
 
 	/* Launch the thread that will handle incoming events */
@@ -309,6 +347,100 @@ void janus_sampleevh_incoming_event(json_t *event) {
 
 }
 
+json_t *janus_sampleevh_handle_request(json_t *request) {
+	if(g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized)) {
+		return NULL;
+	}
+	/* We can use this requests to apply tweaks to the logic */
+	int error_code = 0;
+	char error_cause[512];
+	JANUS_VALIDATE_JSON_OBJECT(request, request_parameters,
+		error_code, error_cause, TRUE,
+		JANUS_SAMPLEEVH_ERROR_MISSING_ELEMENT, JANUS_SAMPLEEVH_ERROR_INVALID_ELEMENT);
+	if(error_code != 0)
+		goto plugin_response;
+	/* Get the request */
+	const char *request_text = json_string_value(json_object_get(request, "request"));
+	if(!strcasecmp(request_text, "tweak")) {
+		/* We only support a request to tweak the current settings */
+		JANUS_VALIDATE_JSON_OBJECT(request, tweak_parameters,
+			error_code, error_cause, TRUE,
+			JANUS_SAMPLEEVH_ERROR_MISSING_ELEMENT, JANUS_SAMPLEEVH_ERROR_INVALID_ELEMENT);
+		if(error_code != 0)
+			goto plugin_response;
+		/* Parameters we can change */
+		const char *req_events = NULL, *req_backend = NULL,
+			*req_backend_user = NULL, *req_backend_pwd = NULL;
+		int req_grouping = -1, req_maxretr = -1, req_backoff = -1;
+		/* Events */
+		if(json_object_get(request, "events"))
+			req_events = json_string_value(json_object_get(request, "events"));
+		/* Grouping */
+		if(json_object_get(request, "grouping"))
+			req_grouping = json_is_true(json_object_get(request, "grouping"));
+		/* Backend stuff */
+		if(json_object_get(request, "backend"))
+			req_backend = json_string_value(json_object_get(request, "backend"));
+		if(req_backend && strstr(req_backend, "http") != req_backend) {
+			/* Not an HTTP address */
+			error_code = JANUS_SAMPLEEVH_ERROR_INVALID_ELEMENT;
+			g_snprintf(error_cause, sizeof(error_cause), "Invalid HTTP URI '%s'", req_backend);
+			goto plugin_response;
+		}
+		if(json_object_get(request, "backend_user"))
+			req_backend_user = json_string_value(json_object_get(request, "backend_user"));
+		if(json_object_get(request, "backend_pwd"))
+			req_backend_pwd = json_string_value(json_object_get(request, "backend_pwd"));
+		/* Retransmissions stuff */
+		if(json_object_get(request, "max_retransmissions"))
+			req_maxretr = json_integer_value(json_object_get(request, "max_retransmissions"));
+		if(json_object_get(request, "retransmissions_backoff"))
+			req_backoff = json_integer_value(json_object_get(request, "retransmissions_backoff"));
+		/* If we got here, we can enforce */
+		if(req_events)
+			janus_sampleevh_edit_events_mask(req_events);
+		if(req_grouping > -1)
+			group_events = req_grouping ? TRUE : FALSE;
+		if(req_backend || req_backend_user || req_backend_pwd) {
+			janus_mutex_lock(&evh_mutex);
+			if(req_backend) {
+				g_free(backend);
+				backend = g_strdup(req_backend);
+			}
+			if(req_backend_user) {
+				g_free(backend_user);
+				backend_user = g_strdup(req_backend_user);
+			}
+			if(req_backend_pwd) {
+				g_free(backend_pwd);
+				backend_pwd = g_strdup(req_backend_pwd);
+			}
+			janus_mutex_unlock(&evh_mutex);
+		}
+		if(req_maxretr > -1)
+			max_retransmissions = req_maxretr;
+		if(req_backoff > -1)
+			retransmissions_backoff = req_backoff;
+	} else {
+		JANUS_LOG(LOG_VERB, "Unknown request '%s'\n", request_text);
+		error_code = JANUS_SAMPLEEVH_ERROR_INVALID_REQUEST;
+		g_snprintf(error_cause, 512, "Unknown request '%s'", request_text);
+	}
+
+plugin_response:
+		{
+			json_t *response = json_object();
+			if(error_code == 0) {
+				/* Return a success */
+				json_object_set_new(response, "result", json_integer(200));
+			} else {
+				/* Prepare JSON error event */
+				json_object_set_new(response, "error_code", json_integer(error_code));
+				json_object_set_new(response, "error", json_string(error_cause));
+			}
+			return response;
+		}
+}
 
 /* Thread to handle incoming events */
 static void *janus_sampleevh_handler(void *data) {
@@ -530,6 +662,26 @@ static void *janus_sampleevh_handler(void *data) {
 							   }
 							}
 						*/
+					case JANUS_EVENT_TYPE_EXTERNAL:
+						/* This is an external event, not originated by Janus itself
+						 * or any of its plugins, but from an ad-hoc Admin API request
+						 * instead. As such, the content of the event is not bound to
+						 * any rules (apart from the fact that it needs to be a JSON
+						 * object), but can be whatever the external source thought
+						 * appropriate. In order to facilitare life to recipients, all
+						 * external events must contain a "schema" property, which anyway
+						 * is not bound to any rules either. As an example:
+							{
+							   "type": 4,
+							   "timestamp": 28381185382,
+							   "event": {
+								  "schema": "my.custom.source",
+								  "data": {
+								     "whatever": "youwant"
+								  }
+							   }
+							}
+						*/
 						break;
 					default:
 						JANUS_LOG(LOG_WARN, "Unknown type of event '%d'\n", type);
@@ -564,18 +716,20 @@ static void *janus_sampleevh_handler(void *data) {
 			JANUS_LOG(LOG_ERR, "Error initializing CURL context\n");
 			goto done;
 		}
+		janus_mutex_lock(&evh_mutex);
 		curl_easy_setopt(curl, CURLOPT_URL, backend);
+		/* Any credentials? */
+		if(backend_user != NULL && backend_pwd != NULL) {
+			curl_easy_setopt(curl, CURLOPT_USERNAME, backend_user);
+			curl_easy_setopt(curl, CURLOPT_PASSWORD, backend_pwd);
+		}
+		janus_mutex_unlock(&evh_mutex);
 		headers = curl_slist_append(headers, "Accept: application/json");
 		headers = curl_slist_append(headers, "Content-Type: application/json");
 		headers = curl_slist_append(headers, "charsets: utf-8");
 		curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers);
 		curl_easy_setopt(curl, CURLOPT_POSTFIELDS, event_text);
 		curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, janus_sampleehv_write_data);
-		/* Any credentials? */
-		if(backend_user != NULL && backend_pwd != NULL) {
-			curl_easy_setopt(curl, CURLOPT_USERNAME, backend_user);
-			curl_easy_setopt(curl, CURLOPT_PASSWORD, backend_pwd);
-		}
 		/* Don't wait forever (let's say, 10 seconds) */
 		curl_easy_setopt(curl, CURLOPT_TIMEOUT, 10L);
 		/* Send the request */
