@@ -52,7 +52,6 @@ var janus = null;
 var mixertest = null;
 var opaqueId = "audiobridgetest-"+Janus.randomString(12);
 
-var started = false;
 var spinner = null;
 
 var myroom = 1234;	// Demo room
@@ -66,10 +65,7 @@ $(document).ready(function() {
 	// Initialize the library (all console debuggers enabled)
 	Janus.init({debug: "all", callback: function() {
 		// Use a button to start the demo
-		$('#start').click(function() {
-			if(started)
-				return;
-			started = true;
+		$('#start').one('click', function() {
 			$(this).attr('disabled', true).unbind('click');
 			// Make sure the browser supports WebRTC
 			if(!Janus.isWebrtcSupported()) {
@@ -160,17 +156,24 @@ $(document).ready(function() {
 												for(var f in list) {
 													var id = list[f]["id"];
 													var display = list[f]["display"];
+													var setup = list[f]["setup"];
 													var muted = list[f]["muted"];
-													Janus.debug("  >> [" + id + "] " + display + " (muted=" + muted + ")");
+													Janus.debug("  >> [" + id + "] " + display + " (setup=" + setup + ", muted=" + muted + ")");
 													if($('#rp'+id).length === 0) {
 														// Add to the participants list
-														$('#list').append('<li id="rp'+id+'" class="list-group-item">'+display+' <i class="fa fa-microphone-slash"></i></li>');
+														$('#list').append('<li id="rp'+id+'" class="list-group-item">'+display+
+															' <i class="absetup fa fa-chain-broken"></i>' +
+															' <i class="abmuted fa fa-microphone-slash"></i></li>');
 														$('#rp'+id + ' > i').hide();
 													}
 													if(muted === true || muted === "true")
-														$('#rp'+id + ' > i').removeClass('hide').show();
+														$('#rp'+id + ' > i.abmuted').removeClass('hide').show();
 													else
-														$('#rp'+id + ' > i').hide();
+														$('#rp'+id + ' > i.abmuted').hide();
+													if(setup === true || setup === "true")
+														$('#rp'+id + ' > i.absetup').hide();
+													else
+														$('#rp'+id + ' > i.absetup').removeClass('hide').show();
 												}
 											}
 										} else if(event === "roomchanged") {
@@ -186,17 +189,24 @@ $(document).ready(function() {
 												for(var f in list) {
 													var id = list[f]["id"];
 													var display = list[f]["display"];
+													var setup = list[f]["setup"];
 													var muted = list[f]["muted"];
-													Janus.debug("  >> [" + id + "] " + display + " (muted=" + muted + ")");
+													Janus.debug("  >> [" + id + "] " + display + " (setup=" + setup + ", muted=" + muted + ")");
 													if($('#rp'+id).length === 0) {
 														// Add to the participants list
-														$('#list').append('<li id="rp'+id+'" class="list-group-item">'+display+' <i class="fa fa-microphone-slash"></i></li>');
+														$('#list').append('<li id="rp'+id+'" class="list-group-item">'+display+
+															' <i class="absetup fa fa-chain-broken"></i>' +
+															' <i class="abmuted fa fa-microphone-slash"></i></li>');
 														$('#rp'+id + ' > i').hide();
 													}
 													if(muted === true || muted === "true")
-														$('#rp'+id + ' > i').removeClass('hide').show();
+														$('#rp'+id + ' > i.abmuted').removeClass('hide').show();
 													else
-														$('#rp'+id + ' > i').hide();
+														$('#rp'+id + ' > i.abmuted').hide();
+													if(setup === true || setup === "true")
+														$('#rp'+id + ' > i.absetup').hide();
+													else
+														$('#rp'+id + ' > i.absetup').removeClass('hide').show();
 												}
 											}
 										} else if(event === "destroyed") {
@@ -213,17 +223,24 @@ $(document).ready(function() {
 												for(var f in list) {
 													var id = list[f]["id"];
 													var display = list[f]["display"];
+													var setup = list[f]["setup"];
 													var muted = list[f]["muted"];
-													Janus.debug("  >> [" + id + "] " + display + " (muted=" + muted + ")");
+													Janus.debug("  >> [" + id + "] " + display + " (setup=" + setup + ", muted=" + muted + ")");
 													if($('#rp'+id).length === 0) {
 														// Add to the participants list
-														$('#list').append('<li id="rp'+id+'" class="list-group-item">'+display+' <i class="fa fa-microphone-slash"></li>');
+														$('#list').append('<li id="rp'+id+'" class="list-group-item">'+display+
+															' <i class="absetup fa fa-chain-broken"></i>' +
+															' <i class="abmuted fa fa-microphone-slash"></i></li>');
 														$('#rp'+id + ' > i').hide();
 													}
 													if(muted === true || muted === "true")
-														$('#rp'+id + ' > i').removeClass('hide').show();
+														$('#rp'+id + ' > i.abmuted').removeClass('hide').show();
 													else
-														$('#rp'+id + ' > i').hide();
+														$('#rp'+id + ' > i.abmuted').hide();
+													if(setup === true || setup === "true")
+														$('#rp'+id + ' > i.absetup').hide();
+													else
+														$('#rp'+id + ' > i.absetup').removeClass('hide').show();
 												}
 											} else if(msg["error"] !== undefined && msg["error"] !== null) {
 												if(msg["error_code"] === 485) {
@@ -264,10 +281,14 @@ $(document).ready(function() {
 								},
 								onremotestream: function(stream) {
 									$('#room').removeClass('hide').show();
+									var addButtons = false;
 									if($('#roomaudio').length === 0) {
+										addButtons = true;
 										$('#mixedaudio').append('<audio class="rounded centered" id="roomaudio" width="100%" height="100%" autoplay/>');
 									}
 									Janus.attachMediaStream($('#roomaudio').get(0), stream);
+									if(!addButtons)
+										return;
 									// Mute button
 									audioenabled = true;
 									$('#toggleaudio').click(
