@@ -53,8 +53,6 @@ var janus = null;
 var screentest = null;
 var opaqueId = "screensharingtest-"+Janus.randomString(12);
 
-var started = false;
-
 var myusername = null;
 var myid = null;
 
@@ -82,10 +80,7 @@ $(document).ready(function() {
 	// Initialize the library (all console debuggers enabled)
 	Janus.init({debug: "all", callback: function() {
 		// Use a button to start the demo
-		$('#start').click(function() {
-			if(started)
-				return;
-			started = true;
+		$('#start').one('click', function() {
 			$(this).attr('disabled', true).unbind('click');
 			// Make sure the browser supports WebRTC
 			if(!Janus.isWebrtcSupported()) {
@@ -235,18 +230,16 @@ $(document).ready(function() {
 										$('#screencapture').append('<video class="rounded centered" id="screenvideo" width="100%" height="100%" autoplay muted="muted"/>');
 									}
 									Janus.attachMediaStream($('#screenvideo').get(0), stream);
-									$("#screencapture").parent().block({
-										message: '<b>Publishing...</b>',
-										css: {
-											border: 'none',
-											backgroundColor: 'transparent',
-											color: 'white'
-										}
-									});
-									// Track the onended event on the video track, just in case the user
-									// stops screensharing without closing the PeerConnection too
-									screentest.webrtcStuff.myStream.getVideoTracks()[0].onended = function() {
-										screentest.hangup();
+									if(screentest.webrtcStuff.pc.iceConnectionState !== "completed" &&
+											screentest.webrtcStuff.pc.iceConnectionState !== "connected") {
+										$("#screencapture").parent().block({
+											message: '<b>Publishing...</b>',
+											css: {
+												border: 'none',
+												backgroundColor: 'transparent',
+												color: 'white'
+											}
+										});
 									}
 								},
 								onremotestream: function(stream) {
@@ -355,7 +348,7 @@ function shareScreen() {
 	// Create a new room
 	var desc = $('#desc').val();
 	role = "publisher";
-	var create = { "request": "create", "description": desc, "bitrate": 0, "publishers": 1 };
+	var create = { "request": "create", "description": desc, "bitrate": 500000, "publishers": 1 };
 	screentest.send({"message": create, success: function(result) {
 		var event = result["videoroom"];
 		Janus.debug("Event: " + event);
@@ -468,21 +461,19 @@ function newRemoteFeed(id, display) {
 				// The subscriber stream is recvonly, we don't expect anything here
 			},
 			onremotestream: function(stream) {
-				if($('#screenvideo').length > 0) {
-					// Been here already
-					return;
+				if($('#screenvideo').length === 0) {
+					// No remote video yet
+					$('#screencapture').append('<video class="rounded centered" id="waitingvideo" width="100%" height="100%" />');
+					$('#screencapture').append('<video class="rounded centered hide" id="screenvideo" width="100%" height="100%" autoplay/>');
+					// Show the video, hide the spinner and show the resolution when we get a playing event
+					$("#screenvideo").bind("playing", function () {
+						$('#waitingvideo').remove();
+						$('#screenvideo').removeClass('hide');
+						if(spinner !== null && spinner !== undefined)
+							spinner.stop();
+						spinner = null;
+					});
 				}
-				// No remote video yet
-				$('#screencapture').append('<video class="rounded centered" id="waitingvideo" width="100%" height="100%" />');
-				$('#screencapture').append('<video class="rounded centered hide" id="screenvideo" width="100%" height="100%" autoplay/>');
-				// Show the video, hide the spinner and show the resolution when we get a playing event
-				$("#screenvideo").bind("playing", function () {
-					$('#waitingvideo').remove();
-					$('#screenvideo').removeClass('hide');
-					if(spinner !== null && spinner !== undefined)
-						spinner.stop();
-					spinner = null;
-				});
 				Janus.attachMediaStream($('#screenvideo').get(0), stream);
 			},
 			oncleanup: function() {
