@@ -52,8 +52,6 @@ var janus = null;
 var textroom = null;
 var opaqueId = "textroomtest-"+Janus.randomString(12);
 
-var started = false;
-
 var myroom = 1234;	// Demo room
 var myusername = null;
 var myid = null;
@@ -64,10 +62,7 @@ $(document).ready(function() {
 	// Initialize the library (all console debuggers enabled)
 	Janus.init({debug: "all", callback: function() {
 		// Use a button to start the demo
-		$('#start').click(function() {
-			if(started)
-				return;
-			started = true;
+		$('#start').one('click', function() {
 			$(this).attr('disabled', true).unbind('click');
 			// Make sure the browser supports WebRTC
 			if(!Janus.isWebrtcSupported()) {
