@@ -13,6 +13,8 @@
 #include <pthread.h>
 #include <errno.h>
 
+#include "debug.h"
+
 extern int lock_debug;
 
 /*! \brief Janus mutex implementation */
@@ -26,13 +28,13 @@ typedef pthread_mutex_t janus_mutex;
 /*! \brief Janus mutex lock without debug */
 #define janus_mutex_lock_nodebug(a) pthread_mutex_lock(a);
 /*! \brief Janus mutex lock with debug (prints the line that locked a mutex) */
-#define janus_mutex_lock_debug(a) { printf("[%s:%s:%d:] ", __FILE__, __FUNCTION__, __LINE__); printf("LOCK %p\n", a); pthread_mutex_lock(a); };
+#define janus_mutex_lock_debug(a) { JANUS_PRINT("[%s:%s:%d:lock] %p\n", __FILE__, __FUNCTION__, __LINE__, a); pthread_mutex_lock(a); };
 /*! \brief Janus mutex lock wrapper (selective locking debug) */
 #define janus_mutex_lock(a) { if(!lock_debug) { janus_mutex_lock_nodebug(a); } else { janus_mutex_lock_debug(a); } };
 /*! \brief Janus mutex unlock without debug */
 #define janus_mutex_unlock_nodebug(a) pthread_mutex_unlock(a);
 /*! \brief Janus mutex unlock with debug (prints the line that unlocked a mutex) */
-#define janus_mutex_unlock_debug(a) { printf("[%s:%s:%d:] ", __FILE__, __FUNCTION__, __LINE__); printf("UNLOCK %p\n", a); pthread_mutex_unlock(a); };
+#define janus_mutex_unlock_debug(a) { JANUS_PRINT("[%s:%s:%d:unlock] %p\n", __FILE__, __FUNCTION__, __LINE__, a); pthread_mutex_unlock(a); };
 /*! \brief Janus mutex unlock wrapper (selective locking debug) */
 #define janus_mutex_unlock(a) { if(!lock_debug) { janus_mutex_unlock_nodebug(a); } else { janus_mutex_unlock_debug(a); } };
 
