@@ -87,11 +87,6 @@ void janus_sctp_handle_remote_error_event(struct sctp_remote_error *sre);
 void janus_sctp_handle_send_failed_event(struct sctp_send_failed_event *ssfe);
 void janus_sctp_handle_notification(janus_sctp_association *sctp, union sctp_notification *notif, size_t n);
 
-void *janus_sctp_thread(void *data);
-janus_sctp_message *janus_sctp_message_create(gboolean incoming, char *buffer, size_t length);
-void janus_sctp_message_destroy(janus_sctp_message *message);
-static janus_sctp_message exit_message;
-
 static gboolean sctp_running;
 int janus_sctp_init(void) {
 	/* Initialize the SCTP stack */
@@ -112,24 +107,45 @@ void janus_sctp_deinit(void) {
 	sctp_running = FALSE;
 }
 
-janus_sctp_association *janus_sctp_association_create(void *dtls, uint64_t handle_id, uint16_t udp_port) {
-	if(dtls == NULL || udp_port == 0)
+static void janus_sctp_association_free(const janus_refcount *sctp_ref) {
+	janus_sctp_association *sctp = janus_refcount_containerof(sctp_ref, janus_sctp_association, ref);
+	/* This association can be destroyed, free all the resources */
+	janus_refcount_decrease(&sctp->handle->ref);
+	janus_refcount_decrease(&sctp->dtls->ref);
+#ifdef DEBUG_SCTP
+	if(sctp->debug_dump != NULL)
+		fclose(sctp->debug_dump);
+	sctp->debug_dump = NULL;
+#endif
+	g_free(sctp->buffer);
+	g_free(sctp);
+	sctp = NULL;
+}
+
+janus_sctp_association *janus_sctp_association_create(janus_dtls_srtp *dtls, janus_ice_handle *handle, uint16_t udp_port) {
+	if(dtls == NULL || handle == NULL || udp_port == 0)
 		return NULL;
 	
 	/* usrsctp provides UDP encapsulation of SCTP, but we need these messages to
 	 * be encapsulated in DTLS and actually sent/received by libnice, and not by
 	 * usrsctp itself... as such, we make use of the AF_CONN approach */
 
-	janus_sctp_association *sctp = (janus_sctp_association *)g_malloc0(sizeof(janus_sctp_association));
-	if(sctp == NULL) {
-		JANUS_LOG(LOG_FATAL, "Memory error!\n");
-		return NULL;
-	}
+	janus_sctp_association *sctp = g_malloc0(sizeof(janus_sctp_association));
+	janus_refcount_init(&sctp->ref, janus_sctp_association_free);
+	g_atomic_int_set(&sctp->destroyed, 0);
 	sctp->dtls = dtls;
-	sctp->handle_id = handle_id;
+	janus_refcount_increase(&dtls->ref);
+	sctp->handle = handle;
+	janus_refcount_increase(&handle->ref);
+	sctp->handle_id = handle->handle_id;
 	sctp->local_port = 5000;	/* FIXME We always use this one */
 	sctp->remote_port = udp_port;
-	sctp->sock = NULL;
+	sctp->buffer = NULL;
+	sctp->buflen = 0;
+	sctp->offset = 0;
+#ifdef DEBUG_SCTP
+	sctp->debug_dump = NULL;
+#endif
 
 	struct socket *sock = NULL;
 	unsigned int i = 0;
@@ -154,14 +170,19 @@ janus_sctp_association *janus_sctp_association_create(void *dtls, uint64_t handl
 	}
 	sctp->stream_buffer_counter = 0;
 	sctp->sock = NULL;
-	janus_mutex_init(&sctp->mutex);
 
 	usrsctp_register_address((void *)sctp);
 	usrsctp_sysctl_set_sctp_ecn_enable(0);
 	if((sock = usrsctp_socket(AF_CONN, SOCK_STREAM, IPPROTO_SCTP, janus_sctp_incoming_data, NULL, 0, (void *)sctp)) == NULL) {
-		JANUS_LOG(LOG_ERR, "[%"SCNu64"] Error creating usrsctp socket... (%d)\n", handle_id, errno);
-		g_free(sctp);
-		sctp = NULL;
+		JANUS_LOG(LOG_ERR, "[%"SCNu64"] Error creating usrsctp socket... (%d)\n", sctp->handle_id, errno);
+		janus_refcount_decrease(&sctp->ref);
+		return NULL;
+	}
+	/* Make the socket non-blocking. Connect, close, shutdown etc will not block
+	 * the thread waiting for the socket operation to complete. */
+	if (usrsctp_set_non_blocking(sock, 1) < 0) {
+		JANUS_LOG(LOG_ERR, "[%"SCNu64"] Error setting socket to non-blocking... (%d)\n", sctp->handle_id, errno);
+		janus_refcount_decrease(&sctp->ref);
 		return NULL;
 	}
 	/* Set SO_LINGER */
@@ -169,9 +190,8 @@ janus_sctp_association *janus_sctp_association_create(void *dtls, uint64_t handl
 	linger_opt.l_onoff = 1;
 	linger_opt.l_linger = 0;
 	if(usrsctp_setsockopt(sock, SOL_SOCKET, SO_LINGER, &linger_opt, sizeof(linger_opt))) {
-		JANUS_LOG(LOG_ERR, "[%"SCNu64"] setsockopt error: SO_LINGER (%d)\n", handle_id, errno);
-		g_free(sctp);
-		sctp = NULL;
+		JANUS_LOG(LOG_ERR, "[%"SCNu64"] setsockopt error: SO_LINGER (%d)\n", sctp->handle_id, errno);
+		janus_refcount_decrease(&sctp->ref);
 		return NULL;
 	}
 	/* Allow resetting streams */
@@ -179,17 +199,15 @@ janus_sctp_association *janus_sctp_association_create(void *dtls, uint64_t handl
 	av.assoc_id = SCTP_ALL_ASSOC;
 	av.assoc_value = 1;
 	if(usrsctp_setsockopt(sock, IPPROTO_SCTP, SCTP_ENABLE_STREAM_RESET, &av, sizeof(struct sctp_assoc_value)) < 0) {
-		JANUS_LOG(LOG_ERR, "[%"SCNu64"] setsockopt error: SCTP_ENABLE_STREAM_RESET (%d)\n", handle_id, errno);
-		g_free(sctp);
-		sctp = NULL;
+		JANUS_LOG(LOG_ERR, "[%"SCNu64"] setsockopt error: SCTP_ENABLE_STREAM_RESET (%d)\n", sctp->handle_id, errno);
+		janus_refcount_decrease(&sctp->ref);
 		return NULL;
 	}
 	/* Disable Nagle */
 	uint32_t nodelay = 1;
 	if(usrsctp_setsockopt(sock, IPPROTO_SCTP, SCTP_NODELAY, &nodelay, sizeof(nodelay))) {
-		JANUS_LOG(LOG_ERR, "[%"SCNu64"] setsockopt error: SCTP_NODELAY (%d)\n", handle_id, errno);
-		g_free(sctp);
-		sctp = NULL;
+		JANUS_LOG(LOG_ERR, "[%"SCNu64"] setsockopt error: SCTP_NODELAY (%d)\n", sctp->handle_id, errno);
+		janus_refcount_decrease(&sctp->ref);
 		return NULL;
 	}	
 	/* Enable the events of interest */
@@ -200,9 +218,8 @@ janus_sctp_association *janus_sctp_association_create(void *dtls, uint64_t handl
 	for(i = 0; i < sizeof(event_types)/sizeof(uint16_t); i++) {
 		event.se_type = event_types[i];
 		if(usrsctp_setsockopt(sock, IPPROTO_SCTP, SCTP_EVENT, &event, sizeof(event)) < 0) {
-			JANUS_LOG(LOG_ERR, "[%"SCNu64"] setsockopt error: SCTP_EVENT (%d)\n", handle_id, errno);
-			g_free(sctp);
-			sctp = NULL;
+			JANUS_LOG(LOG_ERR, "[%"SCNu64"] setsockopt error: SCTP_EVENT (%d)\n", sctp->handle_id, errno);
+			janus_refcount_decrease(&sctp->ref);
 			return NULL;
 		}
 	}
@@ -212,9 +229,8 @@ janus_sctp_association *janus_sctp_association_create(void *dtls, uint64_t handl
 	initmsg.sinit_num_ostreams = 16;	/* What Firefox says in the INIT (Chrome says 1023) */
 	initmsg.sinit_max_instreams = 2048;	/* What both Chrome and Firefox say in the INIT */
 	if(usrsctp_setsockopt(sock, IPPROTO_SCTP, SCTP_INITMSG, &initmsg, sizeof(struct sctp_initmsg)) < 0) {
-		JANUS_LOG(LOG_ERR, "[%"SCNu64"] setsockopt error: SCTP_INITMSG (%d)\n", handle_id, errno);
-		g_free(sctp);
-		sctp = NULL;
+		JANUS_LOG(LOG_ERR, "[%"SCNu64"] setsockopt error: SCTP_INITMSG (%d)\n", sctp->handle_id, errno);
+		janus_refcount_decrease(&sctp->ref);
 		return NULL;
 	}
 	/* Bind our side of the communication, using AF_CONN as we're doing the actual delivery ourselves */
@@ -223,102 +239,83 @@ janus_sctp_association *janus_sctp_association_create(void *dtls, uint64_t handl
 	sconn.sconn_port = htons(sctp->local_port);
 	sconn.sconn_addr = (void *)sctp;
 	if(usrsctp_bind(sock, (struct sockaddr *)&sconn, sizeof(struct sockaddr_conn)) < 0) {
-		JANUS_LOG(LOG_ERR, "[%"SCNu64"] Error binding client on port %"SCNu16" (%d)\n", handle_id, sctp->local_port, errno);
-		g_free(sctp);
-		sctp = NULL;
+		JANUS_LOG(LOG_ERR, "[%"SCNu64"] Error binding client on port %"SCNu16" (%d)\n", sctp->handle_id, sctp->local_port, errno);
+		janus_refcount_decrease(&sctp->ref);
 		return NULL;
 	}
 
 #ifdef DEBUG_SCTP
 	char debug_file[1024];
-	g_snprintf(debug_file, 1024, "%s/sctp-debug-%"SCNu64".txt", debug_folder, handle_id);
+	g_snprintf(debug_file, 1024, "%s/sctp-debug-%"SCNu64".txt", debug_folder, sctp->handle_id);
 	sctp->debug_dump = fopen(debug_file, "wt");
 #endif
 
-	/* We're done for now, the setup is done elsewhere */
-	janus_mutex_lock(&sctp->mutex);
-	sctp->sock = sock;
-	sctp->messages = g_async_queue_new_full((GDestroyNotify) janus_sctp_message_destroy);
-	sctp->buffer = NULL;
-	sctp->buflen = 0;
-	sctp->offset = 0;
-	GError *error = NULL;
-	char tname[16];
-	g_snprintf(tname, sizeof(tname), "sctp %"SCNu64, sctp->handle_id);
-	sctp->thread = g_thread_try_new(tname, &janus_sctp_thread, sctp, &error);
-	if(error != NULL) {
-		/* Something went wrong... */
-		JANUS_LOG(LOG_ERR, "[%"SCNu64"] Got error %d (%s) trying to launch the SCTP thread...\n", handle_id, error->code, error->message ? error->message : "??");
-		g_async_queue_unref(sctp->messages);
-		sctp->messages = NULL;
-		g_free(sctp);
-		sctp = NULL;
+	/* Operating as client */
+	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Connecting the SCTP association\n", sctp->handle_id);
+	struct sockaddr_conn rconn;
+	memset(&rconn, 0, sizeof(struct sockaddr_conn));
+	rconn.sconn_family = AF_CONN;
+	rconn.sconn_port = htons(sctp->remote_port);
+	rconn.sconn_addr = (void *)sctp;
+#ifdef HAVE_SCONN_LEN
+	rconn.sconn_len = sizeof(struct sockaddr_conn);
+#endif
+	int res = usrsctp_connect(sock, (struct sockaddr *)&rconn, sizeof(struct sockaddr_conn));
+	if(res < 0 && errno != EINPROGRESS) {
+		JANUS_LOG(LOG_ERR, "[%"SCNu64"] Error connecting to SCTP server at port %"SCNu16" (%d)\n", sctp->handle_id, sctp->remote_port, errno);
+		janus_refcount_decrease(&sctp->ref);
 		return NULL;
 	}
-	janus_mutex_unlock(&sctp->mutex);
+	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Connected to the DataChannel peer\n", sctp->handle_id);
+	sctp->sock = sock;
 	return sctp;
 }
 
-int janus_sctp_association_setup(janus_sctp_association *sctp) {
-	if(sctp == NULL)
-		return -1;
-
-	struct socket *sock = sctp->sock;
-	struct sockaddr_conn sconn;
-
-	/* Operating as client */
-	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Connecting the SCTP association\n", sctp->handle_id);
-	memset(&sconn, 0, sizeof(struct sockaddr_conn));
-	sconn.sconn_family = AF_CONN;
-	sconn.sconn_port = htons(sctp->remote_port);
-	sconn.sconn_addr = (void *)sctp;
-#ifdef HAVE_SCONN_LEN
-	sconn.sconn_len = sizeof(struct sockaddr_conn);
-#endif
-	if(usrsctp_connect(sock, (struct sockaddr *)&sconn, sizeof(struct sockaddr_conn)) < 0) {
-		JANUS_LOG(LOG_ERR, "[%"SCNu64"] Error connecting to SCTP server at port %"SCNu16" (%d)\n", sctp->handle_id, sctp->remote_port, errno);
-		return -1;
-	}
-	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Connected to the DataChannel peer\n", sctp->handle_id);
-
-	janus_mutex_lock(&sctp->mutex);
-	sctp->sock = sock;
-	janus_mutex_unlock(&sctp->mutex);
-	return 0;
-}
-
 void janus_sctp_association_destroy(janus_sctp_association *sctp) {
-	if(sctp == NULL)
+	if(sctp == NULL || !g_atomic_int_compare_and_exchange(&sctp->destroyed, 0, 1))
 		return;
-	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Destroying SCTP association\n", sctp->handle_id);
+
 	usrsctp_deregister_address(sctp);
 	usrsctp_shutdown(sctp->sock, SHUT_RDWR);
 	usrsctp_close(sctp->sock);
-	janus_mutex_lock(&sctp->mutex);
-	sctp->dtls = NULL;	/* This will get rid of the thread */
-	g_async_queue_push(sctp->messages, &exit_message);
-	janus_mutex_unlock(&sctp->mutex);
+	janus_refcount_decrease(&sctp->ref);
 }
 
 void janus_sctp_data_from_dtls(janus_sctp_association *sctp, char *buf, int len) {
-	if(sctp == NULL || sctp->dtls == NULL || buf == NULL || len <= 0)
+	if(sctp == NULL || sctp->handle == NULL || buf == NULL || len <= 0)
 		return;
 	JANUS_LOG(LOG_HUGE, "[%"SCNu64"] Data from DTLS to SCTP stack: %d bytes\n", sctp->handle_id, len);
-	janus_mutex_lock(&sctp->mutex);
-	if(sctp->messages != NULL)
-		g_async_queue_push(sctp->messages, janus_sctp_message_create(TRUE, buf, len));
-	janus_mutex_unlock(&sctp->mutex);
+#ifdef DEBUG_SCTP
+	if(sctp->debug_dump != NULL) {
+		/* Dump incoming message */
+		char *dump = usrsctp_dumppacket(buf, len, SCTP_DUMP_INBOUND);
+		if(dump != NULL) {
+			fwrite(dump, sizeof(char), strlen(dump), sctp->debug_dump);
+			fflush(sctp->debug_dump);
+			usrsctp_freedumpbuffer(dump);
+		}
+	}
+#endif
+	usrsctp_conninput((void *)sctp, buf, len, 0);
 }
 
 int janus_sctp_data_to_dtls(void *instance, void *buffer, size_t length, uint8_t tos, uint8_t set_df) {
 	janus_sctp_association *sctp = (janus_sctp_association *)instance;
-	if(sctp == NULL || sctp->dtls == NULL)
+	if(sctp == NULL || sctp->handle == NULL)
 		return -1;
 	JANUS_LOG(LOG_HUGE, "[%"SCNu64"] Data from SCTP to DTLS stack: %zu bytes\n", sctp->handle_id, length);
-	janus_mutex_lock(&sctp->mutex);
-	if(sctp->messages != NULL)
-		g_async_queue_push(sctp->messages, janus_sctp_message_create(FALSE, buffer, length));
-	janus_mutex_unlock(&sctp->mutex);
+#ifdef DEBUG_SCTP
+	if(sctp->debug_dump != NULL) {
+		/* Dump outgoing message */
+		char *dump = usrsctp_dumppacket(buffer, length, SCTP_DUMP_OUTBOUND);
+		if(dump != NULL) {
+			fwrite(dump, sizeof(char), strlen(dump), sctp->debug_dump);
+			fflush(sctp->debug_dump);
+			usrsctp_freedumpbuffer(dump);
+		}
+	}
+#endif
+	janus_ice_relay_sctp(sctp->handle, buffer, length);
 	return 0;
 }
 
@@ -340,7 +337,10 @@ static int janus_sctp_incoming_data(struct socket *sock, union sctp_sockstore ad
 void janus_sctp_send_data(janus_sctp_association *sctp, char *buf, int len) {
 	if(sctp == NULL || buf == NULL || len <= 0)
 		return;
-	JANUS_LOG(LOG_VERB, "[%"SCNu64"] SCTP data to send (%d bytes) coming from a plugin: %.*s\n", sctp->handle_id, len, len, buf);
+	JANUS_LOG(LOG_VERB, "[%"SCNu64"] SCTP data to send (%d bytes) coming from a plugin.\n",
+		  sctp->handle_id, len);
+	JANUS_LOG(LOG_HUGE, "[%"SCNu64"] Outgoing SCTP contents: %.*s\n",
+		  sctp->handle_id, len, buf);
 	/* FIXME Is there any open channel we can use? */
 	int i = 0, found = 0;
 	for(i = 0; i < NUMBER_OF_CHANNELS; i++) {
@@ -473,7 +473,6 @@ int janus_sctp_send_open_request_message(struct socket *sock, uint16_t stream, u
 	JANUS_LOG(LOG_VERB, "Using label '%s' (%zu, %u with padding)\n", label, strlen(label), label_size);
 
 	req = g_malloc0(sizeof(janus_datachannel_open_request) + label_size);
-	memset(req, 0, sizeof(janus_datachannel_open_request) + label_size);
 	req->msg_type = DATA_CHANNEL_OPEN_REQUEST;
 	switch (pr_policy) {
 		case SCTP_PR_SCTP_NONE:
@@ -712,11 +711,7 @@ void janus_sctp_send_outgoing_stream_reset(janus_sctp_association *sctp) {
 		return;
 	}
 	len = sizeof(sctp_assoc_t) + (2 + sctp->stream_buffer_counter) * sizeof(uint16_t);
-	srs = (struct sctp_reset_streams *)g_malloc0(len);
-	if(srs == NULL) {
-		return;
-	}
-	memset(srs, 0, len);
+	srs = g_malloc0(len);
 	srs->srs_flags = SCTP_STREAM_RESET_OUTGOING;
 	srs->srs_number_streams = sctp->stream_buffer_counter;
 	for(i = 0; i < sctp->stream_buffer_counter; i++) {
@@ -829,15 +824,14 @@ void janus_sctp_handle_open_request_message(janus_sctp_association *sctp, janus_
 	char *label = NULL;
 	guint len = ntohs(req->label_length);
 	if(len > 0 && len < length) {
-		label = g_malloc0(len+1);
+		label = g_malloc(len+1);
 		memcpy(label, req->label, len);
 		label[len] = '\0'; 
 	}
 	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Opened channel '%s' (id=%"SCNu16") (%d/%d/%d)\n", sctp->handle_id,
 		label ? label : "??",
 		channel->stream, channel->unordered, channel->pr_policy, channel->pr_value);
-	if(label)
-		g_free(label);
+	g_free(label);
 }
 
 void janus_sctp_handle_open_response_message(janus_sctp_association *sctp, janus_datachannel_open_response *rsp, size_t length, uint16_t stream) {
@@ -916,10 +910,12 @@ void janus_sctp_handle_data_message(janus_sctp_association *sctp, char *buffer,
 	} else {
 		/* Assuming DATA_CHANNEL_PPID_DOMSTRING */
 		/* XXX: Protect for non 0 terminated buffer */
-		JANUS_LOG(LOG_VERB, "[%"SCNu64"] Message received of length %zu on channel with id %d: %.*s\n",
-		       sctp->handle_id, length, channel->id, (int)length, buffer);
+		JANUS_LOG(LOG_VERB, "[%"SCNu64"] SCTP data received of length %zu on channel with id %d.\n",
+		       sctp->handle_id, length, channel->id);
+		JANUS_LOG(LOG_HUGE, "[%"SCNu64"] Incoming SCTP contents: %.*s\n",
+		       sctp->handle_id, (int)length, buffer);
 		/* FIXME: notify this to the core */
-		janus_dtls_notify_data((janus_dtls_srtp *)sctp->dtls, buffer, (int)length);
+		janus_dtls_notify_data(sctp->dtls, buffer, (int)length);
 	}
 	return;
 }
@@ -1260,116 +1256,4 @@ void janus_sctp_handle_notification(janus_sctp_association *sctp, union sctp_not
 	}
 }
 
-
-void *janus_sctp_thread(void *data) {
-	janus_sctp_association *sctp = (janus_sctp_association *)data;
-	if(sctp == NULL) {
-		JANUS_LOG(LOG_ERR, "Invalid SCTP association, closing thread\n");
-		g_thread_unref(g_thread_self());
-		return NULL;
-	}
-	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Starting thread for SCTP association\n", sctp->handle_id);
-	janus_sctp_message *message = NULL;
-	gboolean sent_data = FALSE;
-	while(sctp->dtls != NULL && sctp_running) {
-		/* Anything to do at all? */
-		message = g_async_queue_pop(sctp->messages);
-		if(message == NULL)
-			continue;
-		if(message == &exit_message)
-			break;
-		janus_mutex_lock(&sctp->mutex);
-		if(sctp->dtls == NULL) {
-			/* No DTLS stack anymore, we're done */
-			janus_sctp_message_destroy(message);
-			message = NULL;
-			janus_mutex_unlock(&sctp->mutex);
-			break;
-		}
-		/* Check incoming/outgoing messages */
-		if(!message->incoming) {
-#ifdef DEBUG_SCTP
-			if(sctp->debug_dump != NULL) {
-				/* Dump outgoing message */
-				char *dump = usrsctp_dumppacket(message->buffer, message->length, SCTP_DUMP_OUTBOUND);
-				if(dump != NULL) {
-					fwrite(dump, sizeof(char), strlen(dump), sctp->debug_dump);
-					fflush(sctp->debug_dump);
-					usrsctp_freedumpbuffer(dump);
-				}
-			}
-#endif
-			/* Encapsulate this data in DTLS and send it */
-			janus_dtls_send_sctp_data((janus_dtls_srtp *)sctp->dtls, message->buffer, message->length);
-			janus_sctp_message_destroy(message);
-			message = NULL;
-			if(!sent_data)
-				sent_data = TRUE;
-		} else if(message->incoming && sent_data) {
-#ifdef DEBUG_SCTP
-			if(sctp->debug_dump != NULL) {
-				/* Dump incoming message */
-				char *dump = usrsctp_dumppacket(message->buffer, message->length, SCTP_DUMP_INBOUND);
-				if(dump != NULL) {
-					fwrite(dump, sizeof(char), strlen(dump), sctp->debug_dump);
-					fflush(sctp->debug_dump);
-					usrsctp_freedumpbuffer(dump);
-				}
-			}
-#endif
-			/* Pass this data to the SCTP association */
-			janus_mutex_unlock(&sctp->mutex);
-			usrsctp_conninput((void *)sctp, message->buffer, message->length, 0);
-			janus_mutex_lock(&sctp->mutex);
-		}
-		janus_mutex_unlock(&sctp->mutex);
-		janus_sctp_message_destroy(message);
-		message = NULL;
-	}
-	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Leaving SCTP association thread\n", sctp->handle_id);
-	/* This association has been destroyed, wait a bit and then free all the resources */
-	g_usleep (1*G_USEC_PER_SEC);
-	g_async_queue_unref(sctp->messages);
-	sctp->messages = NULL;
-	sctp->thread = NULL;
-#ifdef DEBUG_SCTP
-	if(sctp->debug_dump != NULL)
-		fclose(sctp->debug_dump);
-	sctp->debug_dump = NULL;
-#endif
-	g_free(sctp->buffer);
-	g_free(sctp);
-	sctp = NULL;
-	g_thread_unref(g_thread_self());
-	return NULL;
-}
-
-janus_sctp_message *janus_sctp_message_create(gboolean incoming, char *buffer, size_t length) {
-	if(buffer == NULL || length == 0)
-		return NULL;
-	janus_sctp_message *message = g_malloc0(sizeof(janus_sctp_message));
-	if(message == NULL)
-		return NULL;
-	message->buffer = g_malloc0(length);
-	if(message->buffer == NULL) {
-		g_free(message);
-		message = NULL;
-		return NULL;
-	}
-	memcpy(message->buffer, buffer, length);
-	message->length = length;
-	message->incoming = incoming;
-	return message;
-}
-
-void janus_sctp_message_destroy(janus_sctp_message *message) {
-	if(message == NULL || message == &exit_message)
-		return;
-	if(message->buffer != NULL)
-		g_free(message->buffer);
-	message->buffer = NULL;
-	g_free(message);
-	message = NULL;
-}
-
 #endif
