@@ -30,9 +30,29 @@ const char *janus_preferred_video_codecs[] = {
 };
 uint janus_video_codecs = sizeof(janus_preferred_video_codecs)/sizeof(*janus_preferred_video_codecs);
 
-void janus_sdp_free(janus_sdp *sdp) {
-	if(!sdp)
+/* Reference counters management */
+void janus_sdp_destroy(janus_sdp *sdp) {
+	if(!sdp || !g_atomic_int_compare_and_exchange(&sdp->destroyed, 0, 1))
 		return;
+	janus_refcount_decrease(&sdp->ref);
+}
+
+void janus_sdp_mline_destroy(janus_sdp_mline *m) {
+	if(!m || !g_atomic_int_compare_and_exchange(&m->destroyed, 0, 1))
+		return;
+	janus_refcount_decrease(&m->ref);
+}
+
+void janus_sdp_attribute_destroy(janus_sdp_attribute *a) {
+	if(!a || !g_atomic_int_compare_and_exchange(&a->destroyed, 0, 1))
+		return;
+	janus_refcount_decrease(&a->ref);
+}
+
+/* Internal frees */
+static void janus_sdp_free(const janus_refcount *sdp_ref) {
+	janus_sdp *sdp = janus_refcount_containerof(sdp_ref, janus_sdp, ref);
+	/* This SDP instance can be destroyed, free all the resources */
 	g_free(sdp->o_name);
 	g_free(sdp->o_addr);
 	g_free(sdp->s_name);
@@ -56,24 +76,9 @@ void janus_sdp_free(janus_sdp *sdp) {
 	g_free(sdp);
 }
 
-janus_sdp_mline *janus_sdp_mline_create(janus_sdp_mtype type, guint16 port, const char *proto, janus_sdp_mdirection direction) {
-	janus_sdp_mline *m = g_malloc0(sizeof(janus_sdp_mline));
-	m->type = type;
-	const char *type_str = janus_sdp_mtype_str(type);
-	if(type_str == NULL) {
-		JANUS_LOG(LOG_WARN, "Unknown media type, type_str will have to be set manually\n");
-	} else {
-		m->type_str = g_strdup(type_str);
-	}
-	m->port = port;
-	m->proto = proto ? g_strdup(proto) : NULL;
-	m->direction = direction;
-	return m;
-}
-
-void janus_sdp_mline_destroy(janus_sdp_mline *mline) {
-	if(!mline)
-		return;
+static void janus_sdp_mline_free(const janus_refcount *mline_ref) {
+	janus_sdp_mline *mline = janus_refcount_containerof(mline_ref, janus_sdp_mline, ref);
+	/* This SDP m-line instance can be destroyed, free all the resources */
 	g_free(mline->type_str);
 	g_free(mline->proto);
 	g_free(mline->c_addr);
@@ -92,6 +97,33 @@ void janus_sdp_mline_destroy(janus_sdp_mline *mline) {
 	g_free(mline);
 }
 
+static void janus_sdp_attribute_free(const janus_refcount *attr_ref) {
+	janus_sdp_attribute *attr = janus_refcount_containerof(attr_ref, janus_sdp_attribute, ref);
+	/* This SDP attribute instance can be destroyed, free all the resources */
+	g_free(attr->name);
+	g_free(attr->value);
+	g_free(attr);
+}
+
+
+/* SDP and m-lines/attributes code */
+janus_sdp_mline *janus_sdp_mline_create(janus_sdp_mtype type, guint16 port, const char *proto, janus_sdp_mdirection direction) {
+	janus_sdp_mline *m = g_malloc0(sizeof(janus_sdp_mline));
+	g_atomic_int_set(&m->destroyed, 0);
+	janus_refcount_init(&m->ref, janus_sdp_mline_free);
+	m->type = type;
+	const char *type_str = janus_sdp_mtype_str(type);
+	if(type_str == NULL) {
+		JANUS_LOG(LOG_WARN, "Unknown media type, type_str will have to be set manually\n");
+	} else {
+		m->type_str = g_strdup(type_str);
+	}
+	m->port = port;
+	m->proto = proto ? g_strdup(proto) : NULL;
+	m->direction = direction;
+	return m;
+}
+
 janus_sdp_mline *janus_sdp_mline_find(janus_sdp *sdp, janus_sdp_mtype type) {
 	if(sdp == NULL)
 		return NULL;
@@ -126,9 +158,12 @@ int janus_sdp_mline_remove(janus_sdp *sdp, janus_sdp_mtype type) {
 janus_sdp_attribute *janus_sdp_attribute_create(const char *name, const char *value, ...) {
 	if(!name)
 		return NULL;
-	janus_sdp_attribute *a = g_malloc0(sizeof(janus_sdp_attribute));
+	janus_sdp_attribute *a = g_malloc(sizeof(janus_sdp_attribute));
+	g_atomic_int_set(&a->destroyed, 0);
+	janus_refcount_init(&a->ref, janus_sdp_attribute_free);
 	a->name = g_strdup(name);
 	a->direction = JANUS_SDP_DEFAULT;
+	a->value = NULL;
 	if(value) {
 		char buffer[512];
 		va_list ap;
@@ -140,14 +175,6 @@ janus_sdp_attribute *janus_sdp_attribute_create(const char *name, const char *va
 	return a;
 }
 
-void janus_sdp_attribute_destroy(janus_sdp_attribute *attr) {
-	if(!attr)
-		return;
-	g_free(attr->name);
-	g_free(attr->value);
-	g_free(attr);
-}
-
 int janus_sdp_attribute_add_to_mline(janus_sdp_mline *mline, janus_sdp_attribute *attr) {
 	if(!mline || !attr)
 		return -1;
@@ -223,6 +250,8 @@ janus_sdp *janus_sdp_parse(const char *sdp, char *error, size_t errlen) {
 		return NULL;
 	}
 	janus_sdp *imported = g_malloc0(sizeof(janus_sdp));
+	g_atomic_int_set(&imported->destroyed, 0);
+	janus_refcount_init(&imported->ref, janus_sdp_free);
 	imported->o_ipv4 = TRUE;
 	imported->c_ipv4 = TRUE;
 
@@ -322,6 +351,8 @@ janus_sdp *janus_sdp_parse(const char *sdp, char *error, size_t errlen) {
 					}
 					case 'a': {
 						janus_sdp_attribute *a = g_malloc0(sizeof(janus_sdp_attribute));
+						g_atomic_int_set(&a->destroyed, 0);
+						janus_refcount_init(&a->ref, janus_sdp_attribute_free);
 						line += 2;
 						char *semicolon = strchr(line, ':');
 						if(semicolon == NULL) {
@@ -351,6 +382,8 @@ janus_sdp *janus_sdp_parse(const char *sdp, char *error, size_t errlen) {
 					}
 					case 'm': {
 						janus_sdp_mline *m = g_malloc0(sizeof(janus_sdp_mline));
+						g_atomic_int_set(&m->destroyed, 0);
+						janus_refcount_init(&m->ref, janus_sdp_mline_free);
 						/* Start with media type, port and protocol */
 						char type[32];
 						char proto[64];
@@ -441,19 +474,21 @@ janus_sdp *janus_sdp_parse(const char *sdp, char *error, size_t errlen) {
 						}
 						*semicolon = '\0';
 						mline->b_name = g_strdup(line);
-						mline->b_value = atoi(semicolon+1);
+						mline->b_value = atol(semicolon+1);
 						*semicolon = ':';
 						break;
 					}
 					case 'a': {
 						janus_sdp_attribute *a = g_malloc0(sizeof(janus_sdp_attribute));
+						g_atomic_int_set(&a->destroyed, 0);
+						janus_refcount_init(&a->ref, janus_sdp_attribute_free);
 						line += 2;
 						char *semicolon = strchr(line, ':');
 						if(semicolon == NULL) {
 							/* Is this a media direction attribute? */
 							janus_sdp_mdirection direction = janus_sdp_parse_mdirection(line);
 							if(direction != JANUS_SDP_INVALID) {
-								g_free(a);
+								janus_sdp_attribute_destroy(a);
 								mline->direction = direction;
 								break;
 							}
@@ -505,7 +540,7 @@ janus_sdp *janus_sdp_parse(const char *sdp, char *error, size_t errlen) {
 	if(!success) {
 		if(error)
 			JANUS_LOG(LOG_ERR, "%s\n", error);
-		janus_sdp_free(imported);
+		janus_sdp_destroy(imported);
 		imported = NULL;
 	}
 	return imported;
@@ -634,9 +669,9 @@ const char *janus_sdp_get_codec_name(janus_sdp *sdp, int pt) {
 						return "h264";
 					if(strstr(a->value, "opus") || strstr(a->value, "OPUS"))
 						return "opus";
-					if(strstr(a->value, "pcmu") || strstr(a->value, "PMCU"))
+					if(strstr(a->value, "pcmu") || strstr(a->value, "PCMU"))
 						return "pcmu";
-					if(strstr(a->value, "pcma") || strstr(a->value, "PMCA"))
+					if(strstr(a->value, "pcma") || strstr(a->value, "PCMA"))
 						return "pcma";
 					if(strstr(a->value, "g722") || strstr(a->value, "G722"))
 						return "g722";
@@ -687,7 +722,8 @@ const char *janus_sdp_get_codec_rtpmap(const char *codec) {
 char *janus_sdp_write(janus_sdp *imported) {
 	if(!imported)
 		return NULL;
-	char *sdp = g_malloc0(JANUS_BUFSIZE), buffer[512];
+	janus_refcount_increase(&imported->ref);
+	char *sdp = g_malloc(JANUS_BUFSIZE), buffer[512];
 	*sdp = '\0';
 	/* v= */
 	g_snprintf(buffer, sizeof(buffer), "v=%d\r\n", imported->version);
@@ -764,7 +800,7 @@ char *janus_sdp_write(janus_sdp *imported) {
 		if(m->port > 0) {
 			/* b= */
 			if(m->b_name != NULL) {
-				g_snprintf(buffer, sizeof(buffer), "b=%s:%d\r\n", m->b_name, m->b_value);
+				g_snprintf(buffer, sizeof(buffer), "b=%s:%"SCNu32"\r\n", m->b_name, m->b_value);
 				g_strlcat(sdp, buffer, JANUS_BUFSIZE);
 			}
 		}
@@ -792,12 +828,14 @@ char *janus_sdp_write(janus_sdp *imported) {
 		}
 		temp = temp->next;
 	}
+	janus_refcount_decrease(&imported->ref);
 	return sdp;
 }
 
 void janus_sdp_find_preferred_codecs(janus_sdp *sdp, const char **acodec, const char **vcodec) {
 	if(sdp == NULL)
 		return;
+	janus_refcount_increase(&sdp->ref);
 	gboolean audio = FALSE, video = FALSE;
 	GList *temp = sdp->m_lines;
 	while(temp) {
@@ -832,11 +870,13 @@ void janus_sdp_find_preferred_codecs(janus_sdp *sdp, const char **acodec, const
 			break;
 		temp = temp->next;
 	}
+	janus_refcount_decrease(&sdp->ref);
 }
 
 void janus_sdp_find_first_codecs(janus_sdp *sdp, const char **acodec, const char **vcodec) {
 	if(sdp == NULL)
 		return;
+	janus_refcount_increase(&sdp->ref);
 	gboolean audio = FALSE, video = FALSE;
 	GList *temp = sdp->m_lines;
 	while(temp) {
@@ -869,6 +909,7 @@ void janus_sdp_find_first_codecs(janus_sdp *sdp, const char **acodec, const char
 			break;
 		temp = temp->next;
 	}
+	janus_refcount_decrease(&sdp->ref);
 }
 
 const char *janus_sdp_match_preferred_codec(janus_sdp_mtype type, char *codec) {
@@ -888,7 +929,9 @@ const char *janus_sdp_match_preferred_codec(janus_sdp_mtype type, char *codec) {
 }
 
 janus_sdp *janus_sdp_new(const char *name, const char *address) {
-	janus_sdp *sdp = g_malloc0(sizeof(janus_sdp));
+	janus_sdp *sdp = g_malloc(sizeof(janus_sdp));
+	g_atomic_int_set(&sdp->destroyed, 0);
+	janus_refcount_init(&sdp->ref, janus_sdp_free);
 	/* Fill in some predefined stuff */
 	sdp->version = 0;
 	sdp->o_name = g_strdup("-");
@@ -901,6 +944,8 @@ janus_sdp *janus_sdp_new(const char *name, const char *address) {
 	sdp->t_stop = 0;
 	sdp->c_ipv4 = TRUE;
 	sdp->c_addr = g_strdup(address ? address : "127.0.0.1");
+	sdp->attributes = NULL;
+	sdp->m_lines = NULL;
 	/* Done */
 	return sdp;
 }
@@ -951,6 +996,7 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 	const char *audio_rtpmap = janus_sdp_get_codec_rtpmap(audio_codec);
 	if(do_audio && audio_rtpmap == NULL) {
 		JANUS_LOG(LOG_ERR, "Unsupported audio codec '%s', can't prepare an offer\n", audio_codec);
+		va_end(args);
 		return NULL;
 	}
 	if(video_codec == NULL)
@@ -958,6 +1004,7 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 	const char *video_rtpmap = janus_sdp_get_codec_rtpmap(video_codec);
 	if(do_video && video_rtpmap == NULL) {
 		JANUS_LOG(LOG_ERR, "Unsupported video codec '%s', can't prepare an offer\n", video_codec);
+		va_end(args);
 		return NULL;
 	}
 #ifndef HAVE_SCTP
@@ -1032,6 +1079,7 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 	if(offer == NULL)
 		return NULL;
 
+	janus_refcount_increase(&offer->ref);
 	/* This method has a variable list of arguments, telling us how we should respond */
 	va_list args;
 	va_start(args, offer);
@@ -1071,7 +1119,9 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 	do_data = FALSE;
 #endif
 
-	janus_sdp *answer = g_malloc0(sizeof(janus_sdp));
+	janus_sdp *answer = g_malloc(sizeof(janus_sdp));
+	g_atomic_int_set(&answer->destroyed, 0);
+	janus_refcount_init(&answer->ref, janus_sdp_free);
 	/* Start by copying some of the headers */
 	answer->version = offer->version;
 	answer->o_name = g_strdup(offer->o_name ? offer->o_name : "-");
@@ -1084,6 +1134,8 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 	answer->t_stop = 0;
 	answer->c_ipv4 = offer->c_ipv4;
 	answer->c_addr = g_strdup(offer->c_addr ? offer->c_addr : "127.0.0.1");
+	answer->attributes = NULL;
+	answer->m_lines = NULL;
 
 	/* Now iterate on all media, and let's see what we should do */
 	int audio = 0, video = 0, data = 0;
@@ -1092,6 +1144,8 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 		janus_sdp_mline *m = (janus_sdp_mline *)temp->data;
 		/* For each m-line we parse, we'll need a corresponding one in the answer */
 		janus_sdp_mline *am = g_malloc0(sizeof(janus_sdp_mline));
+		g_atomic_int_set(&am->destroyed, 0);
+		janus_refcount_init(&am->ref, janus_sdp_mline_free);
 		am->type = m->type;
 		am->type_str = m->type_str ? g_strdup(m->type_str) : NULL;
 		am->proto = g_strdup(m->proto ? m->proto : "UDP/TLS/RTP/SAVPF");
@@ -1144,7 +1198,7 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 						am->direction = JANUS_SDP_SENDONLY;
 					} else {
 						/* Peer is recvonly, but we're not ok to send, so reply with inactive */
-						JANUS_LOG(LOG_WARN, "%s offered as '%s', but we need '%s': using 'inactive'\n",
+						JANUS_LOG(LOG_WARN, "%s offered as '%s', but we need '%s' for us: using 'inactive'\n",
 							m->type == JANUS_SDP_AUDIO ? "Audio" : "Video",
 							janus_sdp_mdirection_str(m->direction), janus_sdp_mdirection_str(target_dir));
 						am->direction = JANUS_SDP_INACTIVE;
@@ -1156,7 +1210,7 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 						am->direction = JANUS_SDP_RECVONLY;
 					} else {
 						/* Peer is sendonly, but we're not ok to receive, so reply with inactive */
-						JANUS_LOG(LOG_WARN, "%s offered as '%s', but we need '%s': using 'inactive'\n",
+						JANUS_LOG(LOG_WARN, "%s offered as '%s', but we need '%s' for us: using 'inactive'\n",
 							m->type == JANUS_SDP_AUDIO ? "Audio" : "Video",
 							janus_sdp_mdirection_str(m->direction), janus_sdp_mdirection_str(target_dir));
 						am->direction = JANUS_SDP_INACTIVE;
@@ -1258,6 +1312,9 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 					am->attributes = g_list_append(am->attributes, a);
 					a = janus_sdp_attribute_create("rtcp-fb", "%d goog-remb", pt);
 					am->attributes = g_list_append(am->attributes, a);
+					/* It is safe to add transport-wide rtcp feedback mesage here, won't be used unless the header extension is negotiated*/
+					a = janus_sdp_attribute_create("rtcp-fb", "%d transport-cc", pt);
+					am->attributes = g_list_append(am->attributes, a);
 				}
 			}
 		} else {
@@ -1273,6 +1330,7 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 		}
 		temp = temp->next;
 	}
+	janus_refcount_decrease(&offer->ref);
 
 	/* Done */
 	va_end(args);
