@@ -19,6 +19,8 @@
 #include <inttypes.h>
 #include <glib.h>
 
+#include "refcount.h"
+
 /*! \brief Janus SDP internal object representation */
 typedef struct janus_sdp {
 	/*! \brief v= */
@@ -47,6 +49,10 @@ typedef struct janus_sdp {
 	GList *attributes;
 	/*! \brief List of m= m-lines */
 	GList *m_lines;
+	/*! \brief Atomic flag to check if this instance has been destroyed */
+	volatile gint destroyed;
+	/*! \brief Reference counter for this instance */
+	janus_refcount ref;
 } janus_sdp;
 
 /*! \brief Helper enumeration to quickly identify m-line media types */
@@ -139,11 +145,15 @@ typedef struct janus_sdp_mline {
 	/*! \brief Media b= type */
 	char *b_name;
 	/*! \brief Media b= value */
-	int b_value;
+	uint32_t b_value;
 	/*! \brief Media direction */
 	janus_sdp_mdirection direction;
 	/*! \brief List of m-line attributes */
 	GList *attributes;
+	/*! \brief Atomic flag to check if this instance has been destroyed */
+	volatile gint destroyed;
+	/*! \brief Reference counter for this instance */
+	janus_refcount ref;
 } janus_sdp_mline;
 /*! \brief Helper method to quickly create a janus_sdp_mline instance
  * @note The \c type_str property of the new m-line is created automatically
@@ -184,6 +194,10 @@ typedef struct janus_sdp_attribute {
 	char *value;
 	/*! \brief Attribute direction (e.g., for extmap) */
 	janus_sdp_mdirection direction;
+	/*! \brief Atomic flag to check if this instance has been destroyed */
+	volatile gint destroyed;
+	/*! \brief Reference counter for this instance */
+	janus_refcount ref;
 } janus_sdp_attribute;
 /*! \brief Helper method to quickly create a janus_sdp_attribute instance
  * @param[in] name Name of the attribute
@@ -227,9 +241,9 @@ char *janus_sdp_write(janus_sdp *sdp);
  * @returns A pointer to a janus_sdp object, if successful, NULL otherwise */
 janus_sdp *janus_sdp_new(const char *name, const char *address);
 
-/*! \brief Method to free a Janus SDP object
+/*! \brief Method to destroy a Janus SDP object
  * @param[in] sdp The Janus SDP object to free */
-void janus_sdp_free(janus_sdp *sdp);
+void janus_sdp_destroy(janus_sdp *sdp);
 
 /*! \brief When generating an offer or answer automatically, accept/reject audio if offered (depends on value that follows) */
 #define JANUS_SDP_OA_AUDIO					1
