@@ -55,9 +55,10 @@ const char *janus_mqtt_get_author(void);
 const char *janus_mqtt_get_package(void);
 gboolean janus_mqtt_is_janus_api_enabled(void);
 gboolean janus_mqtt_is_admin_api_enabled(void);
-int janus_mqtt_send_message(void *context, void *request_id, gboolean admin, json_t *message);
-void janus_mqtt_session_created(void *context, guint64 session_id);
-void janus_mqtt_session_over(void *context, guint64 session_id, gboolean timeout);
+int janus_mqtt_send_message(janus_transport_session *transport, void *request_id, gboolean admin, json_t *message);
+void janus_mqtt_session_created(janus_transport_session *transport, guint64 session_id);
+void janus_mqtt_session_over(janus_transport_session *transport, guint64 session_id, gboolean timeout, gboolean claimed);
+void janus_mqtt_session_claimed(janus_transport_session *transport, guint64 session_id);
 
 /* Transport setup */
 static janus_transport janus_mqtt_transport_ =
@@ -79,6 +80,7 @@ static janus_transport janus_mqtt_transport_ =
 		.send_message = janus_mqtt_send_message,
 		.session_created = janus_mqtt_session_created,
 		.session_over = janus_mqtt_session_over,
+		.session_claimed = janus_mqtt_session_claimed,
 	);
 
 /* Transport creator */
@@ -128,6 +130,12 @@ typedef struct janus_mqtt_context {
 			int qos;
 		} publish;
 	} admin;
+	/* SSL config, if needed */
+	gboolean ssl_enable;
+	char *cacert_file;
+	char *cert_file;
+	char *key_file;
+	gboolean verify_peer;
 } janus_mqtt_context;
 
 /* Transport client methods */
@@ -157,6 +165,7 @@ void janus_mqtt_client_destroy_context(janus_mqtt_context **ctx);
 
 /* We only handle a single client */
 static janus_mqtt_context *context_ = NULL;
+static janus_transport_session *mqtt_session = NULL;
 
 int janus_mqtt_init(janus_transport_callbacks *callback, const char *config_path) {
 	if(callback == NULL || config_path == NULL) {
@@ -168,6 +177,8 @@ int janus_mqtt_init(janus_transport_callbacks *callback, const char *config_path
 	janus_mqtt_context *ctx = g_malloc0(sizeof(struct janus_mqtt_context));
 	ctx->gateway = callback;
 	context_ = ctx;
+	/* Prepare the transport session (again, just one) */
+	mqtt_session = janus_transport_session_create(context_, NULL);
 
 	/* Read configuration */
 	char filename[255];
@@ -187,7 +198,7 @@ int janus_mqtt_init(janus_transport_callbacks *callback, const char *config_path
 
 	janus_config_item *username_item = janus_config_get_item_drilldown(config, "general", "username");
 	ctx->connect.username = g_strdup((username_item && username_item->value) ? username_item->value : "guest");
-	
+
 	janus_config_item *password_item = janus_config_get_item_drilldown(config, "general", "password");
 	ctx->connect.password = g_strdup((password_item && password_item->value) ? password_item->value : "guest");
 
@@ -217,6 +228,44 @@ int janus_mqtt_init(janus_transport_callbacks *callback, const char *config_path
 		JANUS_LOG(LOG_WARN, "Notification of events to handlers disabled for %s\n", JANUS_MQTT_NAME);
 	}
 
+	/* Check if we need to enable SSL support */
+	janus_config_item *ssl = janus_config_get_item_drilldown(config, "general", "ssl_enable");
+	if(ssl && ssl->value && janus_is_true(ssl->value)) {
+		if(strstr(url, "ssl://") != url)
+			JANUS_LOG(LOG_WARN, "SSL enabled, but MQTT url doesn't start with ssl://...\n");
+
+		ctx->ssl_enable = TRUE;
+
+		janus_config_item *cacertfile = janus_config_get_item_drilldown(config, "general", "cacertfile");
+		if(!cacertfile || !cacertfile->value) {
+			JANUS_LOG(LOG_FATAL, "Missing CA certificate for MQTT integration...\n");
+			goto error;
+		}
+		ctx->cacert_file = g_strdup(cacertfile->value);
+
+		janus_config_item *certfile = janus_config_get_item_drilldown(config, "general", "certfile");
+		ctx->cert_file = (certfile && certfile->value) ? g_strdup(certfile->value) : NULL;
+
+		janus_config_item *keyfile = janus_config_get_item_drilldown(config, "general", "keyfile");
+		ctx->key_file = (keyfile && keyfile->value) ? g_strdup(keyfile->value) : NULL;
+
+		if(ctx->cert_file && !ctx->key_file) {
+			JANUS_LOG(LOG_FATAL, "Certificate is set but key isn't for MQTT integration...\n");
+			goto error;
+		}
+		if(!ctx->cert_file && ctx->key_file) {
+			JANUS_LOG(LOG_FATAL, "Key is set but certificate isn't for MQTT integration...\n");
+			goto error;
+		}
+
+		janus_config_item *verify = janus_config_get_item_drilldown(config, "general", "verify_peer");
+		ctx->verify_peer = (verify && verify->value && janus_is_true(verify->value)) ? TRUE : FALSE;
+	} else {
+		JANUS_LOG(LOG_INFO, "MQTT SSL support disabled\n");
+		if(strstr(url, "ssl://") == url)
+			JANUS_LOG(LOG_WARN, "SSL disabled, but MQTT url starts with ssl:// instead of tcp://...\n");
+	}
+
 	/* Connect configuration */
 	janus_config_item *keep_alive_interval_item = janus_config_get_item_drilldown(config, "general", "keep_alive_interval");
 	ctx->connect.keep_alive_interval = (keep_alive_interval_item && keep_alive_interval_item->value) ? atoi(keep_alive_interval_item->value) : 20;
@@ -257,8 +306,7 @@ int janus_mqtt_init(janus_transport_callbacks *callback, const char *config_path
 			janus_config_item *qos_item = janus_config_get_item_drilldown(config, "general", "publish_qos");
 			ctx->publish.qos = (qos_item && qos_item->value) ? atoi(qos_item->value) : 1;
 		}
-	}
-	else {
+	} else {
 		janus_mqtt_api_enabled_ = FALSE;
 		ctx->subscribe.topic = NULL;
 		ctx->publish.topic = NULL;
@@ -294,8 +342,7 @@ int janus_mqtt_init(janus_transport_callbacks *callback, const char *config_path
 			janus_config_item *qos_item = janus_config_get_item_drilldown(config, "admin", "publish_qos");
 			ctx->admin.publish.qos = (qos_item && qos_item->value) ? atoi(qos_item->value) : 1;
 		}
-	}
-	else {
+	} else {
 		janus_mqtt_admin_api_enabled_ = FALSE;
 		ctx->admin.subscribe.topic = NULL;
 		ctx->admin.publish.topic = NULL;
@@ -333,14 +380,18 @@ int janus_mqtt_init(janus_transport_callbacks *callback, const char *config_path
 		goto error;
 	}
 
+	g_free((char *)url);
+	g_free((char *)client_id);
+	janus_config_destroy(config);
 	return 0;
 
 error:
 	/* If we got here, something went wrong */
+	janus_transport_session_destroy(mqtt_session);
 	janus_mqtt_client_destroy_context(&ctx);
 	g_free((char *)url);
 	g_free((char *)client_id);
-	g_free(config);
+	janus_config_destroy(config);
 
 	return -1;
 }
@@ -348,6 +399,7 @@ error:
 void janus_mqtt_destroy(void) {
 	JANUS_LOG(LOG_INFO, "Disconnecting MQTT client...\n");
 
+	janus_transport_session_destroy(mqtt_session);
 	janus_mqtt_client_disconnect(context_);
 }
 
@@ -387,16 +439,17 @@ gboolean janus_mqtt_is_admin_api_enabled(void) {
 	return janus_mqtt_admin_api_enabled_;
 }
 
-int janus_mqtt_send_message(void *context, void *request_id, gboolean admin, json_t *message) {
-	if(message == NULL) {
+int janus_mqtt_send_message(janus_transport_session *transport, void *request_id, gboolean admin, json_t *message) {
+	if(message == NULL || transport == NULL) {
 		return -1;
 	}
-	if(context == NULL) {
+	/* Not really needed as we always only have a single context, but that's fine */
+	janus_mqtt_context *ctx = (janus_mqtt_context *)transport->transport_p;
+	if(ctx == NULL) {
 		json_decref(message);
 		return -1;
 	}
 
-	janus_mqtt_context *ctx = (janus_mqtt_context *)context;
 	char *payload = json_dumps(message, json_format_);
 	json_decref(message);
 	JANUS_LOG(LOG_HUGE, "Sending %s API message via MQTT: %s\n", admin ? "admin" : "Janus", payload);
@@ -405,18 +458,24 @@ int janus_mqtt_send_message(void *context, void *request_id, gboolean admin, jso
 	if(rc != MQTTASYNC_SUCCESS) {
 		JANUS_LOG(LOG_ERR, "Can't publish to MQTT topic: %s, return code: %d\n", admin ? ctx->admin.publish.topic : ctx->publish.topic, rc);
 	}
+	free(payload);
 
 	return 0;
 }
 
-void janus_mqtt_session_created(void *context, guint64 session_id) {
+void janus_mqtt_session_created(janus_transport_session *transport, guint64 session_id) {
 	/* We don't care */
 }
 
-void janus_mqtt_session_over(void *context, guint64 session_id, gboolean timeout) {
+void janus_mqtt_session_over(janus_transport_session *transport, guint64 session_id, gboolean timeout, gboolean claimed) {
 	/* We don't care, not even if it's a timeout (should we?), our client is always up */
 }
 
+void janus_mqtt_session_claimed(janus_transport_session *transport, guint64 session_id) {
+	/* We don't care about this. We should start receiving messages from the core about this session: no action necessary */
+	/* FIXME Is the above statement accurate? Should we care? Unlike the HTTP transport, there is no hashtable to update */
+}
+
 void janus_mqtt_client_connection_lost(void *context, char *cause) {
 	JANUS_LOG(LOG_INFO, "MQTT connection lost cause of %s. Reconnecting...\n", cause);
 	/* Automatic reconnect */
@@ -426,7 +485,7 @@ void janus_mqtt_client_connection_lost(void *context, char *cause) {
 	if(notify_events && ctx && ctx->gateway && ctx->gateway->events_is_enabled()) {
 		json_t *info = json_object();
 		json_object_set_new(info, "event", json_string("reconnecting"));
-		ctx->gateway->notify_event(&janus_mqtt_transport_, context, info);
+		ctx->gateway->notify_event(&janus_mqtt_transport_, mqtt_session, info);
 	}
 }
 
@@ -442,7 +501,7 @@ int janus_mqtt_client_message_arrived(void *context, char *topicName, int topicL
 
 		json_error_t error;
 		json_t *root = json_loadb(message->payload, message->payloadlen, 0, &error);
-		ctx->gateway->incoming_request(&janus_mqtt_transport_, ctx, NULL, admin, root, &error);
+		ctx->gateway->incoming_request(&janus_mqtt_transport_, mqtt_session, NULL, admin, root, &error);
 	}
 
 	MQTTAsync_freeMessage(&message);
@@ -462,6 +521,16 @@ int janus_mqtt_client_connect(janus_mqtt_context *ctx) {
 	options.automaticReconnect = TRUE;
 	options.onSuccess = janus_mqtt_client_connect_success;
 	options.onFailure = janus_mqtt_client_connect_failure;
+	/* Is SSL enabled? */
+	MQTTAsync_SSLOptions ssl_opts = MQTTAsync_SSLOptions_initializer;
+	if(ctx->ssl_enable) {
+		ssl_opts.trustStore = ctx->cacert_file;
+		ssl_opts.keyStore = ctx->cert_file;
+		ssl_opts.privateKey = ctx->key_file;
+		ssl_opts.enableServerCertAuth = ctx->verify_peer;
+		options.ssl = &ssl_opts;
+	}
+	/* Connect now */
 	options.context = ctx;
 	return MQTTAsync_connect(ctx->client, &options);
 }
@@ -488,13 +557,13 @@ void janus_mqtt_client_connect_success(void *context, MQTTAsync_successData *res
 	if(notify_events && ctx->gateway && ctx->gateway->events_is_enabled()) {
 		json_t *info = json_object();
 		json_object_set_new(info, "event", json_string("connected"));
-		ctx->gateway->notify_event(&janus_mqtt_transport_, context, info);
+		ctx->gateway->notify_event(&janus_mqtt_transport_, mqtt_session, info);
 	}
 }
 
 void janus_mqtt_client_connect_failure(void *context, MQTTAsync_failureData *response) {
 	int rc = response ? response->code : 0;
-	JANUS_LOG(LOG_ERR, "MQTT client has been failed connecting to the broker, return code: %d. Reconnecting...\n", rc);
+	JANUS_LOG(LOG_ERR, "MQTT client has failed connecting to the broker, return code: %d. Reconnecting...\n", rc);
 	/* Automatic reconnect */
 
 	/* Notify handlers about this transport failure */
@@ -503,7 +572,7 @@ void janus_mqtt_client_connect_failure(void *context, MQTTAsync_failureData *res
 		json_t *info = json_object();
 		json_object_set_new(info, "event", json_string("failed"));
 		json_object_set_new(info, "code", json_integer(rc));
-		ctx->gateway->notify_event(&janus_mqtt_transport_, context, info);
+		ctx->gateway->notify_event(&janus_mqtt_transport_, mqtt_session, info);
 	}
 }
 
@@ -527,7 +596,7 @@ void janus_mqtt_client_reconnect_success(void *context, MQTTAsync_successData *r
 
 void janus_mqtt_client_reconnect_failure(void *context, MQTTAsync_failureData *response) {
 	int rc = response ? response->code : 0;
-	JANUS_LOG(LOG_ERR, "MQTT client has been failed reconnecting from MQTT broker, return code: %d\n", rc);
+	JANUS_LOG(LOG_ERR, "MQTT client has failed reconnecting from MQTT broker, return code: %d\n", rc);
 }
 
 int janus_mqtt_client_disconnect(janus_mqtt_context *ctx) {
@@ -547,7 +616,7 @@ void janus_mqtt_client_disconnect_success(void *context, MQTTAsync_successData *
 	if(notify_events && ctx && ctx->gateway && ctx->gateway->events_is_enabled()) {
 		json_t *info = json_object();
 		json_object_set_new(info, "event", json_string("disconnected"));
-		ctx->gateway->notify_event(&janus_mqtt_transport_, context, info);
+		ctx->gateway->notify_event(&janus_mqtt_transport_, mqtt_session, info);
 	}
 
 	janus_mqtt_client_destroy_context(&ctx);
@@ -568,8 +637,7 @@ int janus_mqtt_client_subscribe(janus_mqtt_context *ctx, gboolean admin) {
 		options.onSuccess = janus_mqtt_client_admin_subscribe_success;
 		options.onFailure = janus_mqtt_client_admin_subscribe_failure;
 		return MQTTAsync_subscribe(ctx->client, ctx->admin.subscribe.topic, ctx->admin.subscribe.qos, &options);
-	}
-	else {
+	} else {
 		options.onSuccess = janus_mqtt_client_subscribe_success;
 		options.onFailure = janus_mqtt_client_subscribe_failure;
 		return MQTTAsync_subscribe(ctx->client, ctx->subscribe.topic, ctx->subscribe.qos, &options);
@@ -592,7 +660,7 @@ void janus_mqtt_client_subscribe_success(void *context, MQTTAsync_successData *r
 void janus_mqtt_client_subscribe_failure(void *context, MQTTAsync_failureData *response) {
 	janus_mqtt_context *ctx = (janus_mqtt_context *)context;
 	int rc = response ? response->code : 0;
-	JANUS_LOG(LOG_ERR, "MQTT client has been failed subscribing to MQTT topic: %s, return code: %d. Reconnecting...\n", ctx->subscribe.topic, rc);
+	JANUS_LOG(LOG_ERR, "MQTT client has failed subscribing to MQTT topic: %s, return code: %d. Reconnecting...\n", ctx->subscribe.topic, rc);
 
 	/* Reconnect */
 	{
@@ -611,7 +679,7 @@ void janus_mqtt_client_admin_subscribe_success(void *context, MQTTAsync_successD
 void janus_mqtt_client_admin_subscribe_failure(void *context, MQTTAsync_failureData *response) {
 	janus_mqtt_context *ctx = (janus_mqtt_context *)context;
 	int rc = response ? response->code : 0;
-	JANUS_LOG(LOG_ERR, "MQTT client has been failed subscribing to MQTT topic: %s, return code: %d. Reconnecting...\n", ctx->admin.subscribe.topic, rc);
+	JANUS_LOG(LOG_ERR, "MQTT client has failed subscribing to MQTT topic: %s, return code: %d. Reconnecting...\n", ctx->admin.subscribe.topic, rc);
 
 	/* Reconnect */
 	{
@@ -635,8 +703,7 @@ int janus_mqtt_client_publish_message(janus_mqtt_context *ctx, char *payload, gb
 		options.onSuccess = janus_mqtt_client_publish_admin_success;
 		options.onFailure = janus_mqtt_client_publish_admin_failure;
 		return MQTTAsync_sendMessage(ctx->client, ctx->admin.publish.topic, &msg, &options);
-	}
-	else {
+	} else {
 		options.onSuccess = janus_mqtt_client_publish_janus_success;
 		options.onFailure = janus_mqtt_client_publish_janus_failure;
 		return MQTTAsync_sendMessage(ctx->client, ctx->publish.topic, &msg, &options);
@@ -651,7 +718,7 @@ void janus_mqtt_client_publish_janus_success(void *context, MQTTAsync_successDat
 void janus_mqtt_client_publish_janus_failure(void *context, MQTTAsync_failureData *response) {
 	janus_mqtt_context *ctx = (janus_mqtt_context *)context;
 	int rc = response ? response->code : 0;
-	JANUS_LOG(LOG_ERR, "MQTT client has been failed publishing to MQTT topic: %s, return code: %d\n", ctx->publish.topic, rc);
+	JANUS_LOG(LOG_ERR, "MQTT client has failed publishing to MQTT topic: %s, return code: %d\n", ctx->publish.topic, rc);
 }
 
 void janus_mqtt_client_publish_admin_success(void *context, MQTTAsync_successData *response) {
@@ -662,7 +729,7 @@ void janus_mqtt_client_publish_admin_success(void *context, MQTTAsync_successDat
 void janus_mqtt_client_publish_admin_failure(void *context, MQTTAsync_failureData *response) {
 	janus_mqtt_context *ctx = (janus_mqtt_context *)context;
 	int rc = response ? response->code : 0;
-	JANUS_LOG(LOG_ERR, "MQTT client has been failed publishing to MQTT topic: %s, return code: %d\n", ctx->admin.publish.topic, rc);
+	JANUS_LOG(LOG_ERR, "MQTT client has failed publishing to MQTT topic: %s, return code: %d\n", ctx->admin.publish.topic, rc);
 }
 
 void janus_mqtt_client_destroy_context(janus_mqtt_context **ptr) {
