@@ -32,6 +32,10 @@
 #include <sys/socket.h>
 #include <poll.h>
 #include <sys/un.h>
+ 
+#ifdef  HAVE_LIBSYSTEMD
+#include "systemd/sd-daemon.h"
+#endif /* HAVE_LIBSYSTEMD */
 
 #include "../debug.h"
 #include "../apierror.h"
@@ -61,9 +65,10 @@ const char *janus_pfunix_get_author(void);
 const char *janus_pfunix_get_package(void);
 gboolean janus_pfunix_is_janus_api_enabled(void);
 gboolean janus_pfunix_is_admin_api_enabled(void);
-int janus_pfunix_send_message(void *transport, void *request_id, gboolean admin, json_t *message);
-void janus_pfunix_session_created(void *transport, guint64 session_id);
-void janus_pfunix_session_over(void *transport, guint64 session_id, gboolean timeout);
+int janus_pfunix_send_message(janus_transport_session *transport, void *request_id, gboolean admin, json_t *message);
+void janus_pfunix_session_created(janus_transport_session *transport, guint64 session_id);
+void janus_pfunix_session_over(janus_transport_session *transport, guint64 session_id, gboolean timeout, gboolean claimed);
+void janus_pfunix_session_claimed(janus_transport_session *transport, guint64 session_id);
 
 
 /* Transport setup */
@@ -86,6 +91,7 @@ static janus_transport janus_pfunix_transport =
 		.send_message = janus_pfunix_send_message,
 		.session_created = janus_pfunix_session_created,
 		.session_over = janus_pfunix_session_over,
+		.session_claimed = janus_pfunix_session_claimed,
 	);
 
 /* Transport creator */
@@ -117,20 +123,39 @@ void *janus_pfunix_thread(void *data);
 /* Unix Sockets servers (and whether they should be SOCK_SEQPACKET or SOCK_DGRAM) */
 static int pfd = -1, admin_pfd = -1;
 static gboolean dgram = FALSE, admin_dgram = FALSE;
+#ifdef HAVE_LIBSYSTEMD
+static gboolean sd_socket = FALSE, admin_sd_socket = FALSE;
+#endif /* HAVE_LIBSYSTEMD */
 /* Socket pair to notify about the need for outgoing data */
 static int write_fd[2];
 
 /* Unix Sockets client session */
 typedef struct janus_pfunix_client {
-	int fd;						/* Client socket (in case SOCK_SEQPACKET is used) */
-	struct sockaddr_un addr;	/* Client address (in case SOCK_DGRAM is used) */
-	gboolean admin;				/* Whether this client is for the Admin or Janus API */
-	GAsyncQueue *messages;		/* Queue of outgoing messages to push */
-	gboolean session_timeout;	/* Whether a Janus session timeout occurred in the core */
+	int fd;							/* Client socket (in case SOCK_SEQPACKET is used) */
+	struct sockaddr_un addr;		/* Client address (in case SOCK_DGRAM is used) */
+	gboolean admin;					/* Whether this client is for the Admin or Janus API */
+	GAsyncQueue *messages;			/* Queue of outgoing messages to push */
+	gboolean session_timeout;		/* Whether a Janus session timeout occurred in the core */
+	janus_transport_session *ts;	/* Janus core-transport session */
 } janus_pfunix_client;
 static GHashTable *clients = NULL, *clients_by_fd = NULL, *clients_by_path = NULL;
 static janus_mutex clients_mutex = JANUS_MUTEX_INITIALIZER;
 
+static void janus_pfunix_client_free(void *client_ref) {
+	if(!client_ref)
+		return;
+	JANUS_LOG(LOG_WARN, "Freeing unix sockets client\n");
+	janus_pfunix_client *client = (janus_pfunix_client *) client_ref;
+	if(client->messages != NULL) {
+		char *response = NULL;
+		while((response = g_async_queue_try_pop(client->messages)) != NULL) {
+			g_free(response);
+		}
+		g_async_queue_unref(client->messages);
+	}
+	g_free(client);
+}
+
 
 /* Helper to create a named Unix Socket out of the path to link to */
 static int janus_pfunix_create_socket(char *pfname, gboolean use_dgram) {
@@ -250,6 +275,12 @@ int janus_pfunix_init(janus_transport_callbacks *callback, const char *config_pa
 				JANUS_LOG(LOG_WARN, "No path configured, skipping Unix Sockets server (Janus API)\n");
 			} else {
 				JANUS_LOG(LOG_INFO, "Configuring %s Unix Sockets server (Janus API)\n", type);
+#ifdef HAVE_LIBSYSTEMD
+				if (sd_listen_fds(0) > 0) {
+					pfd = SD_LISTEN_FDS_START + 0;
+					sd_socket = TRUE;
+				} else
+#endif /* HAVE_LIBSYSTEMD */
 				pfd = janus_pfunix_create_socket(pfname, dgram);
 			}
 		}
@@ -274,6 +305,12 @@ int janus_pfunix_init(janus_transport_callbacks *callback, const char *config_pa
 				JANUS_LOG(LOG_WARN, "No path configured, skipping Unix Sockets server (Admin API)\n");
 			} else {
 				JANUS_LOG(LOG_INFO, "Configuring %s Unix Sockets server (Admin API)\n", type);
+#ifdef HAVE_LIBSYSTEMD
+				if (sd_listen_fds(0) > 1) {
+					admin_pfd = SD_LISTEN_FDS_START + 1;
+					admin_sd_socket = TRUE;
+				} else
+#endif /* HAVE_LIBSYSTEMD */
 				admin_pfd = janus_pfunix_create_socket(pfname, admin_dgram);
 			}
 		}
@@ -363,15 +400,15 @@ gboolean janus_pfunix_is_admin_api_enabled(void) {
 	return admin_pfd > -1;
 }
 
-int janus_pfunix_send_message(void *transport, void *request_id, gboolean admin, json_t *message) {
+int janus_pfunix_send_message(janus_transport_session *transport, void *request_id, gboolean admin, json_t *message) {
 	if(message == NULL)
 		return -1;
-	if(transport == NULL) {
+	if(transport == NULL || transport->transport_p == NULL) {
 		json_decref(message);
 		return -1;
 	}
 	/* Make sure this is related to a still valid Unix Sockets session */
-	janus_pfunix_client *client = (janus_pfunix_client *)transport;
+	janus_pfunix_client *client = (janus_pfunix_client *)transport->transport_p;
 	janus_mutex_lock(&clients_mutex);
 	if(g_hash_table_lookup(clients, client) == NULL) {
 		janus_mutex_unlock(&clients_mutex);
@@ -403,16 +440,16 @@ int janus_pfunix_send_message(void *transport, void *request_id, gboolean admin,
 	return 0;
 }
 
-void janus_pfunix_session_created(void *transport, guint64 session_id) {
+void janus_pfunix_session_created(janus_transport_session *transport, guint64 session_id) {
 	/* We don't care */
 }
 
-void janus_pfunix_session_over(void *transport, guint64 session_id, gboolean timeout) {
+void janus_pfunix_session_over(janus_transport_session *transport, guint64 session_id, gboolean timeout, gboolean claimed) {
 	/* We only care if it's a timeout: if so, close the connection */
-	if(transport == NULL || !timeout)
+	if(transport == NULL || transport->transport_p == NULL || !timeout)
 		return;
 	/* FIXME Should we really close the connection in case of a timeout? */
-	janus_pfunix_client *client = (janus_pfunix_client *)transport;
+	janus_pfunix_client *client = (janus_pfunix_client *)transport->transport_p;
 	janus_mutex_lock(&clients_mutex);
 	if(g_hash_table_lookup(clients, client) != NULL) {
 		client->session_timeout = TRUE;
@@ -425,6 +462,11 @@ void janus_pfunix_session_over(void *transport, guint64 session_id, gboolean tim
 	janus_mutex_unlock(&clients_mutex);
 }
 
+void janus_pfunix_session_claimed(janus_transport_session *transport, guint64 session_id) {
+	/* We don't care about this. We should start receiving messages from the core about this session: no action necessary */
+	/* FIXME Is the above statement accurate? Should we care? Unlike the HTTP transport, there is no hashtable to update */
+}
+
 
 /* Thread */
 void *janus_pfunix_thread(void *data) {
@@ -529,12 +571,12 @@ void *janus_pfunix_thread(void *data) {
 					}
 					JANUS_LOG(LOG_INFO, "Unix Sockets client disconnected (%d)\n", poll_fds[i].fd);
 					/* Notify core */
-					gateway->transport_gone(&janus_pfunix_transport, client);
+					gateway->transport_gone(&janus_pfunix_transport, client->ts);
 					/* Notify handlers about this transport being gone */
 					if(notify_events && gateway->events_is_enabled()) {
 						json_t *info = json_object();
 						json_object_set_new(info, "event", json_string("disconnected"));
-						gateway->notify_event(&janus_pfunix_transport, client, info);
+						gateway->notify_event(&janus_pfunix_transport, client->ts, info);
 					}
 					/* Close socket */
 					shutdown(SHUT_RDWR, poll_fds[i].fd);
@@ -543,14 +585,8 @@ void *janus_pfunix_thread(void *data) {
 					/* Destroy the client */
 					g_hash_table_remove(clients_by_fd, GINT_TO_POINTER(poll_fds[i].fd));
 					g_hash_table_remove(clients, client);
-					if(client->messages != NULL) {
-						char *response = NULL;
-						while((response = g_async_queue_try_pop(client->messages)) != NULL) {
-							g_free(response);
-						}
-						g_async_queue_unref(client->messages);
-					}
-					g_free(client);
+					/* Unref the transport instance */
+					janus_transport_session_destroy(client->ts);
 					janus_mutex_unlock(&clients_mutex);
 					continue;
 				}
@@ -608,11 +644,14 @@ void *janus_pfunix_thread(void *data) {
 							JANUS_LOG(LOG_INFO, "Got new Unix Sockets %s API client: %d\n",
 								poll_fds[i].fd == pfd ? "Janus" : "Admin", cfd);
 							/* Allocate new client */
-							janus_pfunix_client *client = g_malloc0(sizeof(janus_pfunix_client));
+							janus_pfunix_client *client = g_malloc(sizeof(janus_pfunix_client));
 							client->fd = cfd;
+							memset(&client->addr, 0, sizeof(client->addr));
 							client->admin = (poll_fds[i].fd == admin_pfd);	/* API client type */
 							client->messages = g_async_queue_new();
 							client->session_timeout = FALSE;
+							/* Create a transport instance as well */
+							client->ts = janus_transport_session_create(client, janus_pfunix_client_free);
 							/* Take note of this new client */
 							janus_mutex_lock(&clients_mutex);
 							g_hash_table_insert(clients_by_fd, GINT_TO_POINTER(cfd), client);
@@ -624,7 +663,7 @@ void *janus_pfunix_thread(void *data) {
 								json_object_set_new(info, "event", json_string("connected"));
 								json_object_set_new(info, "admin_api", client->admin ? json_true() : json_false());
 								json_object_set_new(info, "fd", json_integer(client->fd));
-								gateway->notify_event(&janus_pfunix_transport, client, info);
+								gateway->notify_event(&janus_pfunix_transport, client->ts, info);
 							}
 						}
 					} else {
@@ -652,12 +691,14 @@ void *janus_pfunix_thread(void *data) {
 							JANUS_LOG(LOG_INFO, "Got new Unix Sockets %s API client: %s\n",
 								poll_fds[i].fd == pfd ? "Janus" : "Admin", uaddr->sun_path);
 							/* Allocate new client */
-							client = g_malloc0(sizeof(janus_pfunix_client));
+							client = g_malloc(sizeof(janus_pfunix_client));
 							client->fd = -1;
 							memcpy(&client->addr, uaddr, sizeof(struct sockaddr_un));
 							client->admin = (poll_fds[i].fd == admin_pfd);	/* API client type */
 							client->messages = g_async_queue_new();
 							client->session_timeout = FALSE;
+							/* Create a transport instance as well */
+							client->ts = janus_transport_session_create(client, janus_pfunix_client_free);
 							/* Take note of this new client */
 							g_hash_table_insert(clients_by_path, uaddr->sun_path, client);
 							g_hash_table_insert(clients, client, client);
@@ -668,7 +709,7 @@ void *janus_pfunix_thread(void *data) {
 								json_object_set_new(info, "admin_api", client->admin ? json_true() : json_false());
 								json_object_set_new(info, "fd", json_integer(client->fd));
 								json_object_set_new(info, "type", json_string("SOCK_DGRAM"));
-								gateway->notify_event(&janus_pfunix_transport, client, info);
+								gateway->notify_event(&janus_pfunix_transport, client->ts, info);
 							}
 						}
 						janus_mutex_unlock(&clients_mutex);
@@ -678,7 +719,7 @@ void *janus_pfunix_thread(void *data) {
 						json_error_t error;
 						json_t *root = json_loads(buffer, 0, &error);
 						/* Notify the core, passing both the object and, since it may be needed, the error */
-						gateway->incoming_request(&janus_pfunix_transport, client, NULL, client->admin, root, &error);
+						gateway->incoming_request(&janus_pfunix_transport, client->ts, NULL, client->admin, root, &error);
 					}
 				} else {
 					/* Client data: receive message */
@@ -709,12 +750,12 @@ void *janus_pfunix_thread(void *data) {
 					if(res == 0) {
 						JANUS_LOG(LOG_INFO, "Unix Sockets client disconnected (%d)\n", poll_fds[i].fd);
 						/* Notify core */
-						gateway->transport_gone(&janus_pfunix_transport, client);
+						gateway->transport_gone(&janus_pfunix_transport, client->ts);
 						/* Notify handlers about this transport being gone */
 						if(notify_events && gateway->events_is_enabled()) {
 							json_t *info = json_object();
 							json_object_set_new(info, "event", json_string("disconnected"));
-							gateway->notify_event(&janus_pfunix_transport, client, info);
+							gateway->notify_event(&janus_pfunix_transport, client->ts, info);
 						}
 						/* Close socket */
 						shutdown(SHUT_RDWR, poll_fds[i].fd);
@@ -723,14 +764,8 @@ void *janus_pfunix_thread(void *data) {
 						/* Destroy the client */
 						g_hash_table_remove(clients_by_fd, GINT_TO_POINTER(poll_fds[i].fd));
 						g_hash_table_remove(clients, client);
-						if(client->messages != NULL) {
-							char *response = NULL;
-							while((response = g_async_queue_try_pop(client->messages)) != NULL) {
-								g_free(response);
-							}
-							g_async_queue_unref(client->messages);
-						}
-						g_free(client);
+						/* Unref the transport instance */
+						janus_transport_session_destroy(client->ts);
 						janus_mutex_unlock(&clients_mutex);
 						continue;
 					}
@@ -743,17 +778,21 @@ void *janus_pfunix_thread(void *data) {
 					json_error_t error;
 					json_t *root = json_loads(buffer, 0, &error);
 					/* Notify the core, passing both the object and, since it may be needed, the error */
-					gateway->incoming_request(&janus_pfunix_transport, client, NULL, client->admin, root, &error);
+					gateway->incoming_request(&janus_pfunix_transport, client->ts, NULL, client->admin, root, &error);
 				}
 			}
 		}
 	}
 
 	socklen_t addrlen = sizeof(struct sockaddr_un);
-	void *addr = g_malloc0(addrlen+1);
+	void *addr = g_malloc(addrlen+1);
 	if(pfd > -1) {
 		/* Unlink the path name first */
+#ifdef HAVE_LIBSYSTEMD
+		if((getsockname(pfd, (struct sockaddr *)addr, &addrlen) != -1) && (FALSE == sd_socket)) {
+#else
 		if(getsockname(pfd, (struct sockaddr *)addr, &addrlen) != -1) {
+#endif
 			JANUS_LOG(LOG_INFO, "Unlinking %s\n", ((struct sockaddr_un *)addr)->sun_path);
 			unlink(((struct sockaddr_un *)addr)->sun_path);
 		}
@@ -763,7 +802,11 @@ void *janus_pfunix_thread(void *data) {
 	pfd = -1;
 	if(admin_pfd > -1) {
 		/* Unlink the path name first */
+#ifdef HAVE_LIBSYSTEMD
+		if((getsockname(admin_pfd, (struct sockaddr *)addr, &addrlen) != -1) && (FALSE == admin_sd_socket)) {
+#else
 		if(getsockname(admin_pfd, (struct sockaddr *)addr, &addrlen) != -1) {
+#endif
 			JANUS_LOG(LOG_INFO, "Unlinking %s\n", ((struct sockaddr_un *)addr)->sun_path);
 			unlink(((struct sockaddr_un *)addr)->sun_path);
 		}
