@@ -62,9 +62,10 @@ const char *janus_websockets_get_author(void);
 const char *janus_websockets_get_package(void);
 gboolean janus_websockets_is_janus_api_enabled(void);
 gboolean janus_websockets_is_admin_api_enabled(void);
-int janus_websockets_send_message(void *transport, void *request_id, gboolean admin, json_t *message);
-void janus_websockets_session_created(void *transport, guint64 session_id);
-void janus_websockets_session_over(void *transport, guint64 session_id, gboolean timeout);
+int janus_websockets_send_message(janus_transport_session *transport, void *request_id, gboolean admin, json_t *message);
+void janus_websockets_session_created(janus_transport_session *transport, guint64 session_id);
+void janus_websockets_session_over(janus_transport_session *transport, guint64 session_id, gboolean timeout, gboolean claimed);
+void janus_websockets_session_claimed(janus_transport_session *transport, guint64 session_id);
 
 
 /* Transport setup */
@@ -87,6 +88,7 @@ static janus_transport janus_websockets_transport =
 		.send_message = janus_websockets_send_message,
 		.session_created = janus_websockets_session_created,
 		.session_over = janus_websockets_session_over,
+		.session_claimed = janus_websockets_session_claimed,
 	);
 
 /* Transport creator */
@@ -124,17 +126,13 @@ typedef struct janus_websockets_client {
 	int buflen;								/* Length of the buffer (may be resized after re-allocations) */
 	int bufpending;							/* Data an interrupted previous write couldn't send */
 	int bufoffset;							/* Offset from where the interrupted previous write should resume */
-	janus_mutex mutex;						/* Mutex to lock/unlock this session */
-	gint session_timeout:1;					/* Whether a Janus session timeout occurred in the core */
-	gint destroy:1;							/* Flag to trigger a lazy session destruction */
+	volatile gint destroyed;				/* Whether this libwebsockets client instance has been closed */
+	janus_transport_session *ts;			/* Janus core-transport session */
 } janus_websockets_client;
 
 
 /* libwebsockets WS context */
 static struct lws_context *wsc = NULL;
-/* libwebsockets sessions that have been closed */
-static GList *old_wss;
-static janus_mutex old_wss_mutex = JANUS_MUTEX_INITIALIZER;
 /* Callbacks for HTTP-related events (automatically rejected) */
 static int janus_websockets_callback_http(
 		struct lws *wsi,
@@ -308,6 +306,10 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 		return -1;
 	}
 
+#ifndef LWS_WITH_IPV6
+	JANUS_LOG(LOG_WARN, "libwebsockets has been built without IPv6 support, will bind to IPv4 only\n");
+#endif
+
 	/* This is the callback we'll need to invoke to contact the gateway */
 	gateway = callback;
 
@@ -363,7 +365,6 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 		}
 		JANUS_LOG(LOG_VERB, "libwebsockets logging: %d\n", ws_log_level);
 		lws_set_log_level(ws_log_level, NULL);
-		old_wss = NULL;
 
 		/* Any ACL for either the Janus or Admin API? */
 		item = janus_config_get_item_drilldown(config, "general", "ws_acl");
@@ -481,13 +482,14 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 			info.extensions = NULL;
 			info.ssl_cert_filepath = NULL;
 			info.ssl_private_key_filepath = NULL;
+			info.ssl_private_key_password = NULL;
 			info.gid = -1;
 			info.uid = -1;
 			info.options = 0;
 			/* Create the WebSocket context */
 			wss = lws_create_vhost(wsc, &info);
 			if(wss == NULL) {
-				JANUS_LOG(LOG_FATAL, "Error initializing libwebsockets...\n");
+				JANUS_LOG(LOG_FATAL, "Error creating vhost for WebSockets server...\n");
 			} else {
 				JANUS_LOG(LOG_INFO, "WebSockets server started (port %d)...\n", wsport);
 			}
@@ -521,9 +523,13 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 			} else {
 				char *server_pem = (char *)item->value;
 				char *server_key = (char *)item->value;
+				char *password = NULL;
 				item = janus_config_get_item_drilldown(config, "certificates", "cert_key");
 				if(item && item->value)
 					server_key = (char *)item->value;
+				item = janus_config_get_item_drilldown(config, "certificates", "cert_pwd");
+				if(item && item->value)
+					password = (char *)item->value;
 				JANUS_LOG(LOG_VERB, "Using certificates:\n\t%s\n\t%s\n", server_pem, server_key);
 				/* Prepare secure context */
 				struct lws_context_creation_info info;
@@ -534,6 +540,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				info.extensions = NULL;
 				info.ssl_cert_filepath = server_pem;
 				info.ssl_private_key_filepath = server_key;
+				info.ssl_private_key_password = password;
 				info.gid = -1;
 				info.uid = -1;
 #if LWS_LIBRARY_VERSION_MAJOR >= 2
@@ -544,7 +551,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				/* Create the secure WebSocket context */
 				swss = lws_create_vhost(wsc, &info);
 				if(swss == NULL) {
-					JANUS_LOG(LOG_FATAL, "Error initializing libwebsockets...\n");
+					JANUS_LOG(LOG_FATAL, "Error creating vhost for Secure WebSockets server...\n");
 				} else {
 					JANUS_LOG(LOG_INFO, "Secure WebSockets server started (port %d)...\n", wsport);
 				}
@@ -583,13 +590,14 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 			info.extensions = NULL;
 			info.ssl_cert_filepath = NULL;
 			info.ssl_private_key_filepath = NULL;
+			info.ssl_private_key_password = NULL;
 			info.gid = -1;
 			info.uid = -1;
 			info.options = 0;
 			/* Create the WebSocket context */
 			admin_wss = lws_create_vhost(wsc, &info);
 			if(admin_wss == NULL) {
-				JANUS_LOG(LOG_FATAL, "Error initializing libwebsockets...\n");
+				JANUS_LOG(LOG_FATAL, "Error creating vhost for Admin WebSockets server...\n");
 			} else {
 				JANUS_LOG(LOG_INFO, "Admin WebSockets server started (port %d)...\n", wsport);
 			}
@@ -623,9 +631,13 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 			} else {
 				char *server_pem = (char *)item->value;
 				char *server_key = (char *)item->value;
+				char *password = NULL;
 				item = janus_config_get_item_drilldown(config, "certificates", "cert_key");
 				if(item && item->value)
 					server_key = (char *)item->value;
+				item = janus_config_get_item_drilldown(config, "certificates", "cert_pwd");
+				if(item && item->value)
+					password = (char *)item->value;
 				JANUS_LOG(LOG_VERB, "Using certificates:\n\t%s\n\t%s\n", server_pem, server_key);
 				/* Prepare secure context */
 				struct lws_context_creation_info info;
@@ -636,6 +648,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				info.extensions = NULL;
 				info.ssl_cert_filepath = server_pem;
 				info.ssl_private_key_filepath = server_key;
+				info.ssl_private_key_password = password;
 				info.gid = -1;
 				info.uid = -1;
 #if LWS_LIBRARY_VERSION_MAJOR >= 2
@@ -646,7 +659,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				/* Create the secure WebSocket context */
 				admin_swss = lws_create_vhost(wsc, &info);
 				if(admin_swss == NULL) {
-					JANUS_LOG(LOG_FATAL, "Error initializing libwebsockets...\n");
+					JANUS_LOG(LOG_FATAL, "Error creating vhost for Secure Admin WebSockets server...\n");
 				} else {
 					JANUS_LOG(LOG_INFO, "Secure Admin WebSockets server started (port %d)...\n", wsport);
 				}
@@ -708,25 +721,21 @@ static void janus_websockets_destroy_client(
 		janus_websockets_client *ws_client,
 		struct lws *wsi,
 		const char *log_prefix) {
-	if(!ws_client || ws_client->destroy)
+	if(!ws_client || !g_atomic_int_compare_and_exchange(&ws_client->destroyed, 0, 1))
 		return;
+	/* Cleanup */
+	janus_mutex_lock(&ws_client->ts->mutex);
+	JANUS_LOG(LOG_INFO, "[%s-%p] Destroying WebSocket client\n", log_prefix, wsi);
+	ws_client->wsi = NULL;
 	/* Notify handlers about this transport being gone */
 	if(notify_events && gateway->events_is_enabled()) {
 		json_t *info = json_object();
 		json_object_set_new(info, "event", json_string("disconnected"));
-		gateway->notify_event(&janus_websockets_transport, ws_client, info);
+		gateway->notify_event(&janus_websockets_transport, ws_client->ts, info);
 	}
 	/* Notify core */
-	gateway->transport_gone(&janus_websockets_transport, ws_client);
-	/* Mark the session as closed */
-	janus_mutex_lock(&old_wss_mutex);
-	old_wss = g_list_append(old_wss, ws_client);
-	janus_mutex_unlock(&old_wss_mutex);
-	/* Cleanup */
-	janus_mutex_lock(&ws_client->mutex);
-	JANUS_LOG(LOG_INFO, "[%s-%p] Destroying WebSocket client\n", log_prefix, wsi);
-	ws_client->destroy = 1;
-	ws_client->wsi = NULL;
+	gateway->transport_gone(&janus_websockets_transport, ws_client->ts);
+	ws_client->ts->transport_p = NULL;
 	/* Remove messages queue too, if needed */
 	if(ws_client->messages != NULL) {
 		char *response = NULL;
@@ -743,7 +752,8 @@ static void janus_websockets_destroy_client(
 	ws_client->buflen = 0;
 	ws_client->bufpending = 0;
 	ws_client->bufoffset = 0;
-	janus_mutex_unlock(&ws_client->mutex);
+	janus_mutex_unlock(&ws_client->ts->mutex);
+	janus_transport_session_destroy(ws_client->ts);
 }
 
 int janus_websockets_get_api_compatibility(void) {
@@ -783,52 +793,40 @@ gboolean janus_websockets_is_admin_api_enabled(void) {
 	return ws_admin_api_enabled;
 }
 
-int janus_websockets_send_message(void *transport, void *request_id, gboolean admin, json_t *message) {
+int janus_websockets_send_message(janus_transport_session *transport, void *request_id, gboolean admin, json_t *message) {
 	if(message == NULL)
 		return -1;
-	if(transport == NULL) {
+	if(transport == NULL || g_atomic_int_get(&transport->destroyed)) {
 		json_decref(message);
 		return -1;
 	}
-	/* Make sure this is not related to a closed /freed WebSocket session */
-	janus_mutex_lock(&old_wss_mutex);
-	janus_websockets_client *client = (janus_websockets_client *)transport;
-	if(g_list_find(old_wss, client) != NULL || !client->wsi) {
+	janus_mutex_lock(&transport->mutex);
+	janus_websockets_client *client = (janus_websockets_client *)transport->transport_p;
+	if(!client) {
 		json_decref(message);
-		message = NULL;
-		transport = NULL;
-		janus_mutex_unlock(&old_wss_mutex);
+		janus_mutex_unlock(&transport->mutex);
 		return -1;
 	}
-	janus_mutex_lock(&client->mutex);
 	/* Convert to string and enqueue */
 	char *payload = json_dumps(message, json_format);
 	g_async_queue_push(client->messages, payload);
 	lws_callback_on_writable(client->wsi);
-	janus_mutex_unlock(&client->mutex);
-	janus_mutex_unlock(&old_wss_mutex);
+	janus_mutex_unlock(&transport->mutex);
 	json_decref(message);
 	return 0;
 }
 
-void janus_websockets_session_created(void *transport, guint64 session_id) {
+void janus_websockets_session_created(janus_transport_session *transport, guint64 session_id) {
 	/* We don't care */
 }
 
-void janus_websockets_session_over(void *transport, guint64 session_id, gboolean timeout) {
-	if(transport == NULL || !timeout)
-		return;
-	/* We only care if it's a timeout: if so, close the connection */
-	janus_websockets_client *client = (janus_websockets_client *)transport;
-	/* Make sure this is not related to a closed WebSocket session */
-	janus_mutex_lock(&old_wss_mutex);
-	if(g_list_find(old_wss, client) == NULL && client->wsi){
-		janus_mutex_lock(&client->mutex);
-		client->session_timeout = 1;
-		lws_callback_on_writable(client->wsi);
-		janus_mutex_unlock(&client->mutex);
-	}
-	janus_mutex_unlock(&old_wss_mutex);
+void janus_websockets_session_over(janus_transport_session *transport, guint64 session_id, gboolean timeout, gboolean claimed) {
+	/* We don't care either: transport timeouts can be detected using the ping/pong mechanism */
+}
+
+void janus_websockets_session_claimed(janus_transport_session *transport, guint64 session_id) {
+	/* We don't care about this. We should start receiving messages from the core about this session: no action necessary */
+	/* FIXME Is the above statement accurate? Should we care? Unlike the HTTP transport, there is no hashtable to update */
 }
 
 
@@ -922,11 +920,6 @@ static int janus_websockets_common_callback(
 				JANUS_LOG(LOG_ERR, "[%s-%p] Invalid WebSocket client instance...\n", log_prefix, wsi);
 				return -1;
 			}
-			/* Clean the old sessions list, in case this pointer was used before */
-			janus_mutex_lock(&old_wss_mutex);
-			if(g_list_find(old_wss, ws_client) != NULL)
-				old_wss = g_list_remove(old_wss, ws_client);
-			janus_mutex_unlock(&old_wss_mutex);
 			/* Prepare the session */
 			ws_client->wsi = wsi;
 			ws_client->messages = g_async_queue_new();
@@ -934,9 +927,8 @@ static int janus_websockets_common_callback(
 			ws_client->buflen = 0;
 			ws_client->bufpending = 0;
 			ws_client->bufoffset = 0;
-			ws_client->session_timeout = 0;
-			ws_client->destroy = 0;
-			janus_mutex_init(&ws_client->mutex);
+			g_atomic_int_set(&ws_client->destroyed, 0);
+			ws_client->ts = janus_transport_session_create(ws_client, NULL);
 			/* Let us know when the WebSocket channel becomes writeable */
 			lws_callback_on_writable(wsi);
 			JANUS_LOG(LOG_VERB, "[%s-%p]   -- Ready to be used!\n", log_prefix, wsi);
@@ -946,7 +938,7 @@ static int janus_websockets_common_callback(
 				json_object_set_new(info, "event", json_string("connected"));
 				json_object_set_new(info, "admin_api", admin ? json_true() : json_false());
 				json_object_set_new(info, "ip", json_string(ip));
-				gateway->notify_event(&janus_websockets_transport, ws_client, info);
+				gateway->notify_event(&janus_websockets_transport, ws_client->ts, info);
 			}
 			return 0;
 		}
@@ -956,11 +948,13 @@ static int janus_websockets_common_callback(
 				JANUS_LOG(LOG_ERR, "[%s-%p] Invalid WebSocket client instance...\n", log_prefix, wsi);
 				return -1;
 			}
+			if(g_atomic_int_get(&ws_client->destroyed))
+				return 0;
 			/* Is this a new message, or part of a fragmented one? */
 			const size_t remaining = lws_remaining_packet_payload(wsi);
 			if(ws_client->incoming == NULL) {
 				JANUS_LOG(LOG_HUGE, "[%s-%p] First fragment: %zu bytes, %zu remaining\n", log_prefix, wsi, len, remaining);
-				ws_client->incoming = g_malloc0(len+1);
+				ws_client->incoming = g_malloc(len+1);
 				memcpy(ws_client->incoming, in, len);
 				ws_client->incoming[len] = '\0';
 				JANUS_LOG(LOG_HUGE, "%s\n", ws_client->incoming);
@@ -984,7 +978,7 @@ static int janus_websockets_common_callback(
 			g_free(ws_client->incoming);
 			ws_client->incoming = NULL;
 			/* Notify the core, passing both the object and, since it may be needed, the error */
-			gateway->incoming_request(&janus_websockets_transport, ws_client, NULL, admin, root, &error);
+			gateway->incoming_request(&janus_websockets_transport, ws_client->ts, NULL, admin, root, &error);
 			return 0;
 		}
 		case LWS_CALLBACK_SERVER_WRITEABLE: {
@@ -992,11 +986,11 @@ static int janus_websockets_common_callback(
 				JANUS_LOG(LOG_ERR, "[%s-%p] Invalid WebSocket client instance...\n", log_prefix, wsi);
 				return -1;
 			}
-			if(!ws_client->destroy && !g_atomic_int_get(&stopping)) {
-				janus_mutex_lock(&ws_client->mutex);
+			if(!g_atomic_int_get(&ws_client->destroyed) && !g_atomic_int_get(&stopping)) {
+				janus_mutex_lock(&ws_client->ts->mutex);
 				/* Check if we have a pending/partial write to complete first */
 				if(ws_client->buffer && ws_client->bufpending > 0 && ws_client->bufoffset > 0
-						&& !ws_client->destroy && !g_atomic_int_get(&stopping)) {
+						&& !g_atomic_int_get(&ws_client->destroyed) && !g_atomic_int_get(&stopping)) {
 					JANUS_LOG(LOG_HUGE, "[%s-%p] Completing pending WebSocket write (still need to write last %d bytes)...\n",
 						log_prefix, wsi, ws_client->bufpending);
 					int sent = lws_write(wsi, ws_client->buffer + ws_client->bufoffset, ws_client->bufpending, LWS_WRITE_TEXT);
@@ -1012,20 +1006,15 @@ static int janus_websockets_common_callback(
 					}
 					/* Done for this round, check the next response/notification later */
 					lws_callback_on_writable(wsi);
-					janus_mutex_unlock(&ws_client->mutex);
+					janus_mutex_unlock(&ws_client->ts->mutex);
 					return 0;
 				}
 				/* Shoot all the pending messages */
 				char *response = g_async_queue_try_pop(ws_client->messages);
-				if(response && !ws_client->destroy && !g_atomic_int_get(&stopping)) {
+				if(response && !g_atomic_int_get(&ws_client->destroyed) && !g_atomic_int_get(&stopping)) {
 					/* Gotcha! */
 					int buflen = LWS_SEND_BUFFER_PRE_PADDING + strlen(response) + LWS_SEND_BUFFER_POST_PADDING;
-					if(ws_client->buffer == NULL) {
-						/* Let's allocate a shared buffer */
-						JANUS_LOG(LOG_HUGE, "[%s-%p] Allocating %d bytes (response is %zu bytes)\n", log_prefix, wsi, buflen, strlen(response));
-						ws_client->buflen = buflen;
-						ws_client->buffer = g_malloc0(buflen);
-					} else if(buflen > ws_client->buflen) {
+					if (buflen > ws_client->buflen) {
 						/* We need a larger shared buffer */
 						JANUS_LOG(LOG_HUGE, "[%s-%p] Re-allocating to %d bytes (was %d, response is %zu bytes)\n", log_prefix, wsi, buflen, ws_client->buflen, strlen(response));
 						ws_client->buflen = buflen;
@@ -1046,10 +1035,10 @@ static int janus_websockets_common_callback(
 					free(response);
 					/* Done for this round, check the next response/notification later */
 					lws_callback_on_writable(wsi);
-					janus_mutex_unlock(&ws_client->mutex);
+					janus_mutex_unlock(&ws_client->ts->mutex);
 					return 0;
 				}
-				janus_mutex_unlock(&ws_client->mutex);
+				janus_mutex_unlock(&ws_client->ts->mutex);
 			}
 			return 0;
 		}
