@@ -45,11 +45,6 @@ static size_t janus_turnrest_callback(void *payload, size_t size, size_t nmemb,
 	janus_turnrest_buffer *buf = (struct janus_turnrest_buffer *)data;
 	/* (Re)allocate if needed */
 	buf->buffer = g_realloc(buf->buffer, buf->size+realsize+1);
-	if(buf->buffer == NULL) {
-		/* Memory error! */ 
-		JANUS_LOG(LOG_FATAL, "Memory error!\n");
-		return 0;
-	}
 	/* Update the buffer */
 	memcpy(&(buf->buffer[buf->size]), payload, realsize);
 	buf->size += realsize;
@@ -68,10 +63,8 @@ void janus_turnrest_deinit(void) {
 	/* Cleanup the libcurl initialization */
 	curl_global_cleanup();
 	janus_mutex_lock(&api_mutex);
-	if(api_server != NULL)
-		g_free((char *)api_server);
-	if(api_key != NULL)
-		g_free((char *)api_key);
+	g_free((char *)api_server);
+	g_free((char *)api_key);
 	janus_mutex_unlock(&api_mutex);
 }
 
@@ -79,11 +72,9 @@ void janus_turnrest_set_backend(const char *server, const char *key, const char
 	janus_mutex_lock(&api_mutex);
 	
 	/* Get rid of the old values first */
-	if(api_server != NULL)
-		g_free((char *)api_server);
+	g_free((char *)api_server);
 	api_server = NULL;
-	if(api_key != NULL)
-		g_free((char *)api_key);
+	g_free((char *)api_key);
 	api_key = NULL;
 
 	if(server != NULL) {
@@ -218,7 +209,7 @@ janus_turnrest_response *janus_turnrest_request(void) {
 		return NULL;
 	}
 	/* Turn the response into a janus_turnrest_response object we can use */
-	janus_turnrest_response *response = g_malloc0(sizeof(janus_turnrest_response));
+	janus_turnrest_response *response = g_malloc(sizeof(janus_turnrest_response));
 	response->username = g_strdup(json_string_value(username));
 	response->password = g_strdup(json_string_value(password));
 	response->ttl = ttl ? json_integer_value(ttl) : 0;
@@ -235,7 +226,7 @@ janus_turnrest_response *janus_turnrest_request(void) {
 			JANUS_LOG(LOG_WARN, "Skipping invalid TURN URI '%s' (not a TURN URI)...\n", turn_uri);
 			continue;
 		}
-		janus_turnrest_instance *instance = g_malloc0(sizeof(janus_turnrest_instance));
+		janus_turnrest_instance *instance = g_malloc(sizeof(janus_turnrest_instance));
 		instance->transport = NICE_RELAY_TYPE_TURN_UDP;
 		if(strstr(turn_uri, "turns:") == turn_uri)
 			instance->transport = NICE_RELAY_TYPE_TURN_TLS;
@@ -248,22 +239,22 @@ janus_turnrest_response *janus_turnrest_request(void) {
 		}
 		gchar **uri_parts = g_strsplit(turn_uri, ":", -1);
 		/* Resolve the TURN URI address */
+		struct addrinfo *res = NULL;
 		janus_network_address addr;
 		janus_network_address_string_buffer addr_buf;
-		if(janus_network_string_to_address(janus_network_query_options_any_ip, uri_parts[1], &addr) != 0 ||
+		if(getaddrinfo(uri_parts[1], NULL, NULL, &res) != 0 ||
+				janus_network_address_from_sockaddr(res->ai_addr, &addr) != 0 ||
 				janus_network_address_to_string_buffer(&addr, &addr_buf) != 0) {
 			JANUS_LOG(LOG_WARN, "Skipping invalid TURN URI '%s' (could not resolve the address)...\n", uri_parts[1]);
+			if(res != NULL)
+				freeaddrinfo(res);
 			g_strfreev(uri_parts);
 			continue;
 		}
+		freeaddrinfo(res);
 		instance->server = g_strdup(janus_network_address_string_from_buffer(&addr_buf));
-		if(instance->server == NULL) {
-			JANUS_LOG(LOG_WARN, "Skipping invalid TURN URI '%s' (could not resolve the address)...\n", uri_parts[1]);
-			g_strfreev(uri_parts);
-			continue;
-		}
 		if(uri_parts[2] == NULL) {
-			/* No port? USe 3478 by default */
+			/* No port? Use 3478 by default */
 			instance->port = 3478;
 		} else {
 			instance->port = atoi(uri_parts[2]);
